@@ -1,4 +1,4 @@
 #!/bin/sh
-
-export LD_LIBRARY_PATH=../lib/linux_gtk2
+export MOZILLA_FIVE_HOME=/usr/lib/mozilla
+export LD_LIBRARY_PATH=../lib/linux_gtk2:/usr/lib/mozilla
 gij -Djava.library.path=../lib/linux_gtk2/ -cp ../lib/demos.jar:../lib/linux_gtk2/swt.jar:../lib/linux_gtk2/swt-mozilla.jar:../lib/linux_gtk2/swt-pi.jar:../lib/swingwt.jar demo.SwingWTBrowser $@
