@@ -1,3 +1,4 @@
 #!/bin/sh
-export LD_LIBRARY_PATH=../lib/linux_gtk2/
+export MOZILLA_FIVE_HOME=/usr/lib/mozilla
+export LD_LIBRARY_PATH=../lib/linux_gtk2/:/usr/lib/mozilla
 kaffe -Xss128m -cp ../lib/demos.jar:../lib/linux_gtk2/swt-mozilla.jar:../lib/linux_gtk2/swt.jar:../lib/linux_gtk2/swt-pi.jar:../lib/swingwt.jar:../demo/swingset/swingres.jar demo.swingset.SwingSet2
