@@ -1,2 +1,4 @@
 #!/bin/sh
+export LD_LIBRARY_PATH=/usr/lib/mozilla
+export MOZILLA_FIVE_HOME=/usr/lib/mozilla
 $JAVA_HOME/bin/java -Djava.library.path=../lib/linux_gtk2/ -cp ../lib/demos.jar:../lib/linux_gtk2/swt-mozilla.jar:../lib/linux_gtk2/swt.jar:../lib/linux_gtk2/swt-pi.jar:../lib/swingwt.jar:../demo/swingset/swingres.jar demo.swingset.SwingSet2
