@@ -5,6 +5,22 @@
  * @author R. Rawson-Tetley
  *
  * $Log: Everything.java,v $
+ * Revision 1.53  2004/11/05 11:31:50  bobintetley
+ * getPixmap() variant that uses Class context and relative resources (Shaun Jackman)
+ *
+ * Revision 1.52  2004/11/04 20:49:15  bobintetley
+ * Relative resources breaks under Sun VMs. Gone back to the getPixmap() method
+ *
+ * Revision 1.51  2004/11/04 20:24:20  bobintetley
+ * Whoops, should at least compile before checkin :)
+ *
+ * Revision 1.50  2004/11/04 20:16:02  bobintetley
+ * Relative image resources and moved filechooser images into swingwtx.swing
+ *
+ * Revision 1.49  2004/11/04 10:26:46  bobintetley
+ * getPixmap() call to look for images on the filesystem or as a classpath
+ *    resource.
+ *
  * Revision 1.48  2004/11/02 12:50:09  bobintetley
  * Fix to editorpane display and spinner column size
  *
@@ -94,6 +110,11 @@ public class Everything extends JFrame {
     
     private JPanel comps = null;
     
+    //private ImageIcon imgPic = SwingWTUtils.getPixmap("pic.gif", SwingWTUtils.SWINGWT_PIXMAP_SUBDIR, "/demo/");
+    //private ImageIcon imgComputer = SwingWTUtils.getPixmap("Computer.png", SwingWTUtils.SWINGWT_PIXMAP_SUBDIR, "/demo/");
+    private ImageIcon imgPic = SwingWTUtils.getPixmap(Everything.class, "pic.gif");
+    private ImageIcon imgComputer = SwingWTUtils.getPixmap(Everything.class, "Computer.png");
+    
     public void test() {
         
         setTitle("Test");
@@ -101,7 +122,7 @@ public class Everything extends JFrame {
         setLocation(200, 200);
         
         try {
-            setIconImage(new ImageIcon(getClass().getResource("/demo/pic.gif")).getImage());
+            setIconImage(imgPic.getImage());
         }
         catch (Exception e) {}
         
@@ -151,7 +172,7 @@ public class Everything extends JFrame {
         // Images on buttons
         final JButton btn3 = new JButton("Image Button");
         try {
-            btn3.setIcon(new ImageIcon(getClass().getResource("/demo/Computer.png")));
+            btn3.setIcon(imgComputer);
         }
         catch (Exception e) {}
         btn3.addActionListener(new ActionListener() {
@@ -257,7 +278,7 @@ public class Everything extends JFrame {
                     setBackground(table.getBackground());
                     setForeground(table.getForeground());
                 }
-                setIcon(new ImageIcon(getClass().getResource("/demo/pic.gif")));
+                setIcon(imgPic);
                 setValue(value);
                 return this;
             }
@@ -344,7 +365,7 @@ public class Everything extends JFrame {
                 hasFocus);
                 
                 if (value.toString().indexOf("Root") != -1)
-                    setIcon(new ImageIcon(getClass().getResource("/demo/pic.gif")));
+                    setIcon(imgPic);
                 else
                     setIcon(null);
                 setText(value.toString());
@@ -373,7 +394,7 @@ public class Everything extends JFrame {
         JMenuItem mnuY = new JMenuItem("Tray Popup");
         mnuT.add(mnuY);
         JTaskTrayItem item = new JTaskTrayItem(
-                new ImageIcon(getClass().getResource("/demo/pic.gif")), 
+        		imgPic, 
                 "Sample Task Tray Icon",
                 jpop2);
         item.show();
@@ -399,7 +420,7 @@ public class Everything extends JFrame {
             }
         });
         try {
-            mnuFileOpenDoc.setIcon(new ImageIcon(getClass().getResource("/demo/pic.gif")));
+            mnuFileOpenDoc.setIcon(imgPic);
         }
         catch (Exception e) {}
         
@@ -529,7 +550,7 @@ public class Everything extends JFrame {
         pnl2.setLayout(new GridLayout(2,2));
         
         JButton btnTest = new JButton();
-        btnTest.setIcon(new ImageIcon(getClass().getResource("/demo/Computer.png")));
+        btnTest.setIcon(imgPic);
         pnl2.add(btnTest);
         
         jtp.addTab("Test", pnl1);
@@ -537,7 +558,7 @@ public class Everything extends JFrame {
         
         add(jtp, BorderLayout.CENTER);
         
-        jtp.setIconAt(1, new ImageIcon(getClass().getResource("/demo/Computer.png")));
+        jtp.setIconAt(1, imgComputer);
         
         show();
         dispatchEvents();
