@@ -1,6 +1,6 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
+ * Copyright (c) 2000, 2003 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials 
  * are made available under the terms of the Common Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/cpl-v10.html
@@ -10,12 +10,11 @@
  *******************************************************************************/
 package org.eclipse.swt.dnd;
 
- 
 import org.eclipse.swt.*;
 import org.eclipse.swt.widgets.*;
 import org.eclipse.swt.graphics.*;
-import org.eclipse.swt.internal.*;
-import org.eclipse.swt.internal.gtk.*;
+import org.eclipse.swt.internal.ole.win32.*;
+import org.eclipse.swt.internal.win32.*;
 
 /**
  *
@@ -69,7 +68,7 @@ import org.eclipse.swt.internal.gtk.*;
  * </dl>
  */
 public class DropTarget extends Widget {
-
+	
 	private Control control;
 	private Listener controlListener;
 	private Transfer[] transferAgents = new Transfer[0];
@@ -82,30 +81,16 @@ public class DropTarget extends Widget {
 	// workaround - There is no event for "operation changed" so track operation based on key state
 	private int keyOperation = -1;
 	
-	// workaround - Simulate events when the mouse is not moving
-	private long dragOverStart;
-	private Runnable dragOverHeartbeat;
-	private DNDEvent dragOverEvent;
+	// workaround - The dataobject address is only passed as an argument in drag enter and drop.  
+	// To allow applications to query the data values during the drag over operations, 
+	// maintain a reference to it.
+	private int iDataObject;
 	
-	private int drag_motion_handler;
-	private int drag_leave_handler;
-	private int drag_data_received_handler;
-	private int drag_drop_handler;
+	// interfaces
+	private COMObject iDropTarget;
+	private int refCount;
 	
 	private static final String DROPTARGETID = "DropTarget"; //$NON-NLS-1$
-	private static final int DRAGOVER_HYSTERESIS = 50;
-	
-	private static Callback Drag_Motion;
-	private static Callback Drag_Leave;
-	private static Callback Drag_Data_Received;
-	private static Callback Drag_Drop;
-	
-	 static {	
-		Drag_Motion = new Callback(DropTarget.class, "Drag_Motion", 5); //$NON-NLS-1$
-		Drag_Leave = new Callback(DropTarget.class, "Drag_Leave", 3); //$NON-NLS-1$
-		Drag_Data_Received = new Callback(DropTarget.class, "Drag_Data_Received", 7); //$NON-NLS-1$
-		Drag_Drop = new Callback(DropTarget.class, "Drag_Drop", 5); //$NON-NLS-1$
-	}
 
 /**
  * Creates a new <code>DropTarget</code> to allow data to be dropped on the specified 
@@ -121,11 +106,15 @@ public class DropTarget extends Widget {
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
  *    <li>ERROR_INVALID_SUBCLASS - if this class is not an allowed subclass</li>
+ * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_CANNOT_INIT_DROP - unable to initiate drop target; this will occur if more than one
  *        drop target is created for a control or if the operating system will not allow the creation
  *        of the drop target</li>
  * </ul>
+ *
+ * <p>NOTE: ERROR_CANNOT_INIT_DROP should be an SWTException, since it is a
+ * recoverable error, but can not be changed due to backward compatability.</p>
  * 
  * @see DropTarget#dispose
  * @see DropTarget#checkSubclass
@@ -133,41 +122,35 @@ public class DropTarget extends Widget {
  * @see DND#DROP_COPY
  * @see DND#DROP_MOVE
  * @see DND#DROP_LINK
- */	
+ */
 public DropTarget(Control control, int style) {
-	super(control, checkStyle(style));
+	super (control, checkStyle(style));
 	this.control = control;
-	if (Drag_Motion == null || Drag_Leave == null || Drag_Data_Received == null || Drag_Drop == null) {
-		 DND.error(DND.ERROR_CANNOT_INIT_DROP);
-	}
 	if (control.getData(DROPTARGETID) != null) {
 		DND.error(DND.ERROR_CANNOT_INIT_DROP);
 	}
 	control.setData(DROPTARGETID, this);
+	createCOMInterfaces();
+	this.AddRef();
 	
-	byte[] buffer = Converter.wcsToMbcs(null, "drag_motion", true); //$NON-NLS-1$
-	drag_motion_handler = OS.g_signal_connect(control.handle, buffer, Drag_Motion.getAddress(), 0);
-	buffer = Converter.wcsToMbcs(null, "drag_leave", true); //$NON-NLS-1$
-	drag_leave_handler = OS.g_signal_connect(control.handle, buffer, Drag_Leave.getAddress(), 0);
-	buffer = Converter.wcsToMbcs(null, "drag_data_received", true); //$NON-NLS-1$
-	drag_data_received_handler = OS.g_signal_connect(control.handle, buffer, Drag_Data_Received.getAddress(), 0);
-	buffer = Converter.wcsToMbcs(null, "drag_drop", true); //$NON-NLS-1$
-	drag_drop_handler = OS.g_signal_connect(control.handle, buffer, Drag_Drop.getAddress(), 0);
+	if (COM.CoLockObjectExternal(iDropTarget.getAddress(), true, true) != COM.S_OK)
+		DND.error(DND.ERROR_CANNOT_INIT_DROP);
+	if (COM.RegisterDragDrop( control.handle, iDropTarget.getAddress()) != COM.S_OK)
+		DND.error(DND.ERROR_CANNOT_INIT_DROP);
 
-	// Dispose listeners	
-	controlListener = new Listener(){
-		public void handleEvent(Event event){
+	controlListener = new Listener () {
+		public void handleEvent (Event event) {
 			if (!DropTarget.this.isDisposed()){
 				DropTarget.this.dispose();
 			}
 		}
 	};
-	control.addListener(SWT.Dispose, controlListener);
-		
-	this.addListener(SWT.Dispose, new Listener(){
-		public void handleEvent(Event event){
+	control.addListener (SWT.Dispose, controlListener);
+	
+	this.addListener(SWT.Dispose, new Listener () {
+		public void handleEvent (Event event) {
 			onDispose();
-		}	
+		}
 	});
 
 	// Drag under effect
@@ -178,100 +161,13 @@ public DropTarget(Control control, int style) {
 	} else {
 		effect = new NoDragUnderEffect(control);
 	}
-
-	dragOverHeartbeat = new Runnable() {
-		public void run() {
-			if (DropTarget.this.control.isDisposed() || dragOverStart == 0) return;
-			long time = System.currentTimeMillis();
-			int delay = DRAGOVER_HYSTERESIS;
-			if (time < dragOverStart) {
-				delay = (int)(dragOverStart - time);
-			} else {	
-				int allowedOperations = dragOverEvent.operations;
-				TransferData[] allowedTypes = dragOverEvent.dataTypes;
-				//pass a copy of data types in to listeners in case application modifies it
-				TransferData[] dataTypes = new TransferData[allowedTypes.length];
-				System.arraycopy(allowedTypes, 0, dataTypes, 0, dataTypes.length);
-	
-				DNDEvent event = new DNDEvent();
-				event.widget = dragOverEvent.widget;
-				event.x = dragOverEvent.x;
-				event.y = dragOverEvent.y;
-				event.time = (int)time;
-				event.feedback = DND.FEEDBACK_SELECT;
-				event.dataTypes = dataTypes;
-				event.dataType = selectedDataType;
-				event.operations = dragOverEvent.operations;
-				event.detail  = selectedOperation;
-				
-				try {
-					notifyListeners(DND.DragOver, event);
-				} catch (Throwable e) {
-					event.dataType = null;
-					event.detail  = DND.DROP_NONE;
-				}
-				
-				effect.show(event.feedback, event.x, event.y);
-				
-				selectedDataType = null;
-				if (event.dataType != null) {
-					for (int i = 0; i < allowedTypes.length; i++) {
-						if (allowedTypes[i].type == event.dataType.type) {
-							selectedDataType = event.dataType;
-							break;
-						}
-					}
-				}
-
-				selectedOperation = DND.DROP_NONE;
-				if (selectedDataType != null && (event.detail & allowedOperations) != 0) {
-					selectedOperation = event.detail;
-				}
-			}
-			DropTarget.this.control.getDisplay().timerExec(delay, dragOverHeartbeat);
-		}
-	};
 }
 
 static int checkStyle (int style) {
-	if (style == SWT.NONE) return DND.DROP_MOVE;	
+	if (style == SWT.NONE) return DND.DROP_MOVE;
 	return style;
 }
 
-private static int /*long*/ Drag_Data_Received ( int /*long*/ widget, int /*long*/ context, int /*long*/ x, int /*long*/ y, int /*long*/ data, int /*long*/ info, int /*long*/ time){
-	DropTarget target = FindDropTarget(widget);
-	if (target == null) return 0;
-	target.drag_data_received (widget, context, (int)/*64*/x, (int)/*64*/y, data, (int)/*64*/info, (int)/*64*/time);
-	return 0;
-}
-
-private static int /*long*/ Drag_Drop(int /*long*/ widget, int /*long*/ context, int /*long*/ x, int /*long*/ y, int /*long*/ time) {
-	DropTarget target = FindDropTarget(widget);
-	if (target == null) return 0;
-	return target.drag_drop (widget, context, (int)/*64*/x, (int)/*64*/y, (int)/*64*/time) ? 1 : 0;
-}
-
-private static int /*long*/ Drag_Leave ( int /*long*/ widget, int /*long*/ context, int /*long*/ time){
-	DropTarget target = FindDropTarget(widget);
-	if (target == null) return 0;
-	target.drag_leave (widget, context, (int)/*64*/time);
-	return 0;
-}
-
-private static int /*long*/ Drag_Motion ( int /*long*/ widget, int /*long*/ context, int /*long*/ x, int /*long*/ y, int /*long*/ time){
-	DropTarget target = FindDropTarget(widget);
-	if (target == null) return 0;
-	return target.drag_motion (widget, context, (int)/*64*/x, (int)/*64*/y, (int)/*64*/time) ? 1 : 0;
-}
-	
-private static DropTarget FindDropTarget(int /*long*/ handle) {
-	Display display = Display.findDisplay(Thread.currentThread());
-	if (display == null || display.isDisposed()) return null;
-	Widget widget = display.findWidget(handle);
-	if (widget == null) return null;
-	return (DropTarget)widget.getData(DROPTARGETID);
-}
-
 /**
  * Adds the listener to the collection of listeners who will
  * be notified when a drag and drop operation is in progress, by sending
@@ -304,7 +200,7 @@ private static DropTarget FindDropTarget(int /*long*/ handle) {
  * @see #removeDropListener
  * @see DropTargetEvent
  */
-public void addDropListener(DropTargetListener listener) {	
+public void addDropListener(DropTargetListener listener) {
 	if (listener == null) DND.error (SWT.ERROR_NULL_ARGUMENT);
 	DNDListener typedListener = new DNDListener (listener);
 	addListener (DND.DragEnter, typedListener);
@@ -312,8 +208,13 @@ public void addDropListener(DropTargetListener listener) {
 	addListener (DND.DragOver, typedListener);
 	addListener (DND.DragOperationChanged, typedListener);
 	addListener (DND.Drop, typedListener);
-	addListener (DND.DropAccept, typedListener);	
-}	
+	addListener (DND.DropAccept, typedListener);
+}
+
+private int AddRef() {
+	refCount++;
+	return refCount;
+}
 
 protected void checkSubclass () {
 	String name = getClass().getName ();
@@ -323,198 +224,192 @@ protected void checkSubclass () {
 	}
 }
 
-private void drag_data_received ( int /*long*/ widget, int /*long*/ context, int x, int y, int /*long*/ data, int info, int time){
-	DNDEvent event = new DNDEvent();
-	if (data == 0 || !setEventData(context, x, y, time, event)) {
-		keyOperation = -1;
-		return;
-	}
-	keyOperation = -1;
-	
-	int allowedOperations = event.operations;
-	
-	// Get data in a Java format	
-	Object object = null;
-	TransferData transferData = new TransferData();
-	GtkSelectionData selectionData = new GtkSelectionData(); 
-	OS.memmove(selectionData, data, GtkSelectionData.sizeof);
-	if (selectionData.data != 0) {
-		transferData.type = selectionData.type;
-		transferData.length = selectionData.length;
-		transferData.pValue = selectionData.data;
-		transferData.format = selectionData.format;
-		for (int i = 0; i < transferAgents.length; i++) {
-			if (transferAgents[i].isSupportedType(transferData)) {
-				object = transferAgents[i].nativeToJava(transferData);
-				break;
-			}
-		}
-	}
-	if (object == null) {
-		selectedOperation = DND.DROP_NONE;
-	}
-	
-	event.detail = selectedOperation;
-	event.dataType = transferData;
-	event.data = object;
-	try {
-		notifyListeners(DND.Drop, event);
-		selectedOperation = DND.DROP_NONE;
-		if ((allowedOperations & event.detail) == event.detail) {
-			selectedOperation = event.detail;
-		}
-	} catch (Throwable e) {
-		selectedOperation = DND.DROP_NONE;
-	}
-	
-	//notify source of action taken
-	OS.gtk_drag_finish(context, selectedOperation != DND.DROP_NONE, selectedOperation== DND.DROP_MOVE, time); 			
-	return;	
+private void createCOMInterfaces() {
+	// register each of the interfaces that this object implements
+	iDropTarget = new COMObject(new int[]{2, 0, 0, 5, 4, 0, 5}){
+		public int method0(int[] args) {return QueryInterface(args[0], args[1]);}
+		public int method1(int[] args) {return AddRef();}
+		public int method2(int[] args) {return Release();}
+		public int method3(int[] args) {return DragEnter(args[0], args[1], args[2], args[3], args[4]);}
+		public int method4(int[] args) {return DragOver(args[0], args[1], args[2], args[3]);}
+		public int method5(int[] args) {return DragLeave();}
+		public int method6(int[] args) {return Drop(args[0], args[1], args[2], args[3], args[4]);}
+	};
+
 }
 
-private boolean drag_drop(int /*long*/ widget, int /*long*/ context, int x, int y, int time) {
-	DNDEvent event = new DNDEvent();
-	if (!setEventData(context, x, y, time, event)) {
-		keyOperation = -1;
-		return false;
-	}
-	keyOperation = -1;
-	
-	int allowedOperations = event.operations;
-	TransferData[] allowedDataTypes = new TransferData[event.dataTypes.length];
-	System.arraycopy(event.dataTypes, 0, allowedDataTypes, 0, allowedDataTypes.length);
-	
-	event.dataType = selectedDataType;
-	event.detail = selectedOperation;
-	try {
-		notifyListeners(DND.DropAccept,event);
-	} catch (Throwable err) {
-		event.detail = DND.DROP_NONE;
-		event.dataType = null;
-	}
+private void disposeCOMInterfaces() {
+	if (iDropTarget != null)
+		iDropTarget.dispose();
+	iDropTarget = null;
+}
 
+private int DragEnter(int pDataObject, int grfKeyState, int pt_x, int pt_y, int pdwEffect) {
 	selectedDataType = null;
-	if (event.dataType != null) {
-		for (int i = 0; i < allowedDataTypes.length; i++) {
-			if (allowedDataTypes[i].type == event.dataType.type) {
-				selectedDataType = allowedDataTypes[i];
-				break;
-			}
-		}
-	}
-	
 	selectedOperation = DND.DROP_NONE;
-	if (selectedDataType != null && ((event.detail & allowedOperations) == event.detail)) {
-		selectedOperation = event.detail;
-	}
-	
-	if (selectedOperation == DND.DROP_NONE) {
-		// this was not a successful drop
-		return false;
-	}
-
-	// ask drag source for dropped data
-	OS.gtk_drag_get_data(widget, context, selectedDataType.type, time);
-	return true;
-}
-
-private void drag_leave ( int /*long*/ widget, int /*long*/ context, int time){
-	updateDragOverHover(0, null);
-	effect.show(DND.FEEDBACK_NONE, 0, 0);
-	
-	if (keyOperation == -1) return;
-	keyOperation = -1;
+	iDataObject = 0;
 	
 	DNDEvent event = new DNDEvent();
-	event.widget = this;
-	event.time = time;
-	event.detail = DND.DROP_NONE;
-	try {
-		notifyListeners(DND.DragLeave, event);
-	} catch (Throwable e) {}
-}
+	if (!setEventData(event, pDataObject, grfKeyState, pt_x, pt_y, pdwEffect)) {
+		OS.MoveMemory(pdwEffect, new int[] {COM.DROPEFFECT_NONE}, 4);
+		return COM.S_OK;
+	}	
 
-private boolean drag_motion ( int /*long*/ widget, int /*long*/ context, int x, int y, int time){
-	int oldKeyOperation = keyOperation;
-	
-	if (oldKeyOperation == -1) { //drag enter
-		selectedDataType = null;
-		selectedOperation = DND.DROP_NONE;
-	}
-		
-	DNDEvent event = new DNDEvent();
-	if (!setEventData(context, x, y, time, event)) {
-		keyOperation = -1;
-		OS.gdk_drag_status(context, 0, time);
-		return false;
-	}
+	// Remember the iDataObject because it is not passed into the DragOver callback
+	iDataObject = pDataObject;
 	
 	int allowedOperations = event.operations;
 	TransferData[] allowedDataTypes = new TransferData[event.dataTypes.length];
 	System.arraycopy(event.dataTypes, 0, allowedDataTypes, 0, allowedDataTypes.length);
-
-	if (oldKeyOperation == -1) {
-		event.type = DND.DragEnter;
-	} else {
-		if (keyOperation == oldKeyOperation) {
-			event.type = DND.DragOver;
-			event.dataType = selectedDataType;
-			event.detail = selectedOperation;
-		} else {
-			event.type = DND.DragOperationChanged;
-			event.dataType = selectedDataType;
-		}
-	}
-	
-	updateDragOverHover(DRAGOVER_HYSTERESIS, event);
-						
-	try {
-		notifyListeners(event.type, event);	
-	} catch (Throwable e) {
-		OS.gdk_drag_status(context, 0, time);
-		return false;
-	}
-
+	notifyListeners(DND.DragEnter,event);
 	if (event.detail == DND.DROP_DEFAULT) {
 		event.detail = (allowedOperations & DND.DROP_MOVE) != 0 ? DND.DROP_MOVE : DND.DROP_NONE;
 	}
 	
 	selectedDataType = null;
-	if (event.dataType != null) {
-		for (int i = 0; i < allowedDataTypes.length; i++) {
-			if (allowedDataTypes[i].type == event.dataType.type) {
-				selectedDataType = allowedDataTypes[i];
-				break;
-			}
+	for (int i = 0; i < allowedDataTypes.length; i++) {
+		if (TransferData.sameType(allowedDataTypes[i], event.dataType)){
+			selectedDataType = allowedDataTypes[i];
+			break;
+		}
+	}
+	
+	selectedOperation = DND.DROP_NONE;
+	if (selectedDataType != null && ((allowedOperations & event.detail) != 0)) {
+		selectedOperation = event.detail;
+	}
+
+	effect.show(event.feedback, event.x, event.y);
+	
+	OS.MoveMemory(pdwEffect, new int[] {opToOs(selectedOperation)}, 4);
+	return COM.S_OK;
+}
+
+
+
+private int DragLeave() {
+	effect.show(DND.FEEDBACK_NONE, 0, 0);
+	keyOperation = -1;
+	
+	if (iDataObject == 0) return COM.S_OK;
+
+	DNDEvent event = new DNDEvent();
+	event.widget = this;
+	event.time = OS.GetMessageTime();
+	event.detail = DND.DROP_NONE;
+	notifyListeners(DND.DragLeave, event);
+	return COM.S_OK;
+}
+
+private int DragOver(int grfKeyState, int pt_x,	int pt_y, int pdwEffect) {
+	int oldKeyOperation = keyOperation;
+	
+	DNDEvent event = new DNDEvent();
+	if (!setEventData(event, iDataObject, grfKeyState, pt_x, pt_y, pdwEffect)) {
+		keyOperation = -1;
+		OS.MoveMemory(pdwEffect, new int[] {COM.DROPEFFECT_NONE}, 4);
+		return COM.S_OK;
+	}
+	
+	int allowedOperations = event.operations;
+	TransferData[] allowedDataTypes = new TransferData[event.dataTypes.length];
+	System.arraycopy(event.dataTypes, 0, allowedDataTypes, 0, allowedDataTypes.length);
+
+	if (keyOperation == oldKeyOperation) {
+		event.type = DND.DragOver;
+		event.dataType = selectedDataType;
+		event.detail = selectedOperation;
+	} else {
+		event.type = DND.DragOperationChanged;
+		event.dataType = selectedDataType;
+	}
+	notifyListeners(event.type, event);
+	if (event.detail == DND.DROP_DEFAULT) {
+		event.detail = (allowedOperations & DND.DROP_MOVE) != 0 ? DND.DROP_MOVE : DND.DROP_NONE;
+	}
+	
+	selectedDataType = null;
+	for (int i = 0; i < allowedDataTypes.length; i++) {
+		if (TransferData.sameType(allowedDataTypes[i], event.dataType)){
+			selectedDataType = allowedDataTypes[i];
+			break;
 		}
 	}
 
 	selectedOperation = DND.DROP_NONE;
-	if (selectedDataType != null && (allowedOperations & event.detail) != 0) {
+	if (selectedDataType != null && ((allowedOperations & event.detail) == event.detail)) {
 		selectedOperation = event.detail;
 	}
 	
 	effect.show(event.feedback, event.x, event.y);
+	
+	OS.MoveMemory(pdwEffect, new int[] {opToOs(selectedOperation)}, 4);
+	return COM.S_OK;
+}
 
-	switch (selectedOperation) {
-		case DND.DROP_NONE:
-			OS.gdk_drag_status(context, 0, time);
-			break;		
-		case DND.DROP_COPY:
-			OS.gdk_drag_status(context, OS.GDK_ACTION_COPY, time);
-			break;
-		case DND.DROP_MOVE:
-			OS.gdk_drag_status(context, OS.GDK_ACTION_MOVE, time);
-			break;
-		case DND.DROP_LINK:
-			OS.gdk_drag_status(context, OS.GDK_ACTION_LINK, time);
+private int Drop(int pDataObject, int grfKeyState, int pt_x, int pt_y, int pdwEffect) {
+	effect.show(DND.FEEDBACK_NONE, 0, 0);
+
+	DNDEvent event = new DNDEvent();
+	event.widget = this;
+	event.time = OS.GetMessageTime();
+	event.detail = DND.DROP_NONE;
+	notifyListeners(DND.DragLeave, event);
+	event = new DNDEvent();
+	if (!setEventData(event, pDataObject, grfKeyState, pt_x, pt_y, pdwEffect)) {
+		keyOperation = -1;
+		OS.MoveMemory(pdwEffect, new int[] {COM.DROPEFFECT_NONE}, 4);
+		return COM.S_OK;
+	}
+	keyOperation = -1;
+	
+	int allowedOperations = event.operations;
+	TransferData[] allowedDataTypes = new TransferData[event.dataTypes.length];
+	System.arraycopy(event.dataTypes, 0, allowedDataTypes, 0, allowedDataTypes.length);
+	
+	event.dataType = selectedDataType;		
+	event.detail = selectedOperation;
+	notifyListeners(DND.DropAccept,event);
+	selectedDataType = null;
+	for (int i = 0; i < allowedDataTypes.length; i++) {
+		if (TransferData.sameType(allowedDataTypes[i], event.dataType)){
+			selectedDataType = allowedDataTypes[i];
 			break;
+		}
+	}
+	selectedOperation = DND.DROP_NONE;
+	if (selectedDataType != null && (allowedOperations & event.detail) == event.detail) {
+		selectedOperation = event.detail;
+	}
+
+	if (selectedOperation == DND.DROP_NONE){
+		OS.MoveMemory(pdwEffect, new int[] {COM.DROPEFFECT_NONE}, 4);	
+		return COM.S_OK;
 	}
 	
-	if (oldKeyOperation == -1) {
-		dragOverHeartbeat.run();
+	// Get Data in a Java format
+	Object object = null;
+	for (int i = 0; i < transferAgents.length; i++){
+		if (transferAgents[i].isSupportedType(selectedDataType)){
+			object = transferAgents[i].nativeToJava(selectedDataType);
+			break;
+		}
 	}
-	return true;
+	if (object == null){
+		selectedOperation = DND.DROP_NONE;
+	}
+	
+	event.detail = selectedOperation;
+	event.dataType = selectedDataType;
+	event.data = object;
+	notifyListeners(DND.Drop,event);
+	selectedOperation = DND.DROP_NONE;
+	if ((allowedOperations & event.detail) == event.detail) {
+		selectedOperation = event.detail;
+	}
+	//notify source of action taken		
+	OS.MoveMemory(pdwEffect, new int[] {opToOs(selectedOperation)}, 4);	
+	return COM.S_OK;
 }
 
 /**
@@ -527,11 +422,9 @@ public Control getControl () {
 	return control;
 }
 
-private int getOperationFromKeyState() {
-	int[] state = new int[1];
-	OS.gdk_window_get_pointer(0, null, null, state);
-	boolean ctrl = (state[0] & OS.GDK_CONTROL_MASK) != 0;
-	boolean shift = (state[0] & OS.GDK_SHIFT_MASK) != 0;
+private int getOperationFromKeyState(int grfKeyState) {
+	boolean ctrl = (grfKeyState & OS.MK_CONTROL) != 0;
+	boolean shift = (grfKeyState & OS.MK_SHIFT) != 0;
 	if (ctrl && shift) return DND.DROP_LINK;
 	if (ctrl)return DND.DROP_COPY;
 	if (shift)return DND.DROP_MOVE;
@@ -542,9 +435,9 @@ private int getOperationFromKeyState() {
  * Returns a list of the data types that can be transferred to this DropTarget.
  *
  * @return a list of the data types that can be transferred to this DropTarget
- */ 
-public Transfer[] getTransfer() { 
-	return transferAgents; 
+ */
+public Transfer[] getTransfer() {
+	return transferAgents;
 }
 
 public void notifyListeners (int eventType, Event event) {
@@ -587,44 +480,80 @@ public void notifyListeners (int eventType, Event event) {
 	super.notifyListeners(eventType, event);
 }
 
-private void onDispose(){
+private void onDispose () {	
 	if (control == null) return;
-	OS.g_signal_handler_disconnect(control.handle, drag_motion_handler);
-	OS.g_signal_handler_disconnect(control.handle, drag_leave_handler);
-	OS.g_signal_handler_disconnect(control.handle, drag_data_received_handler);
-	OS.g_signal_handler_disconnect(control.handle, drag_drop_handler);
-	if (transferAgents.length != 0)
-		OS.gtk_drag_dest_unset(control.handle);
-	transferAgents = null;
+
+	COM.RevokeDragDrop(control.handle);
+	
 	if (controlListener != null)
 		control.removeListener(SWT.Dispose, controlListener);
-	control.setData(DROPTARGETID, null);
-	control = null;
 	controlListener = null;
+	control.setData(DROPTARGETID, null);
+	transferAgents = null;
+	control = null;
+	
+	COM.CoLockObjectExternal(iDropTarget.getAddress(), false, true);
+	
+	this.Release();
+	
+	COM.CoFreeUnusedLibraries();
 }
 
-private int opToOsOp(int operation){
+private int opToOs(int operation) {
 	int osOperation = 0;
-	if ((operation & DND.DROP_COPY) == DND.DROP_COPY)
-		osOperation |= OS.GDK_ACTION_COPY;
-	if ((operation & DND.DROP_MOVE) == DND.DROP_MOVE)
-		osOperation |= OS.GDK_ACTION_MOVE;
-	if ((operation & DND.DROP_LINK) == DND.DROP_LINK)
-		osOperation |= OS.GDK_ACTION_LINK;
+	if ((operation & DND.DROP_COPY) != 0){
+		osOperation |= COM.DROPEFFECT_COPY;
+	}
+	if ((operation & DND.DROP_LINK) != 0) {
+		osOperation |= COM.DROPEFFECT_LINK;
+	}
+	if ((operation & DND.DROP_MOVE) != 0) {
+		osOperation |= COM.DROPEFFECT_MOVE;
+	}
 	return osOperation;
 }
 
-private int osOpToOp(int osOperation){
-	int operation = DND.DROP_NONE;
-	if ((osOperation & OS.GDK_ACTION_COPY) == OS.GDK_ACTION_COPY)
+private int osToOp(int osOperation){
+	int operation = 0;
+	if ((osOperation & COM.DROPEFFECT_COPY) != 0){
 		operation |= DND.DROP_COPY;
-	if ((osOperation & OS.GDK_ACTION_MOVE) == OS.GDK_ACTION_MOVE)
-		operation |= DND.DROP_MOVE;
-	if ((osOperation & OS.GDK_ACTION_LINK) == OS.GDK_ACTION_LINK)
+	}
+	if ((osOperation & COM.DROPEFFECT_LINK) != 0) {
 		operation |= DND.DROP_LINK;
+	}
+	if ((osOperation & COM.DROPEFFECT_MOVE) != 0) {
+		operation |= DND.DROP_MOVE;
+	}
 	return operation;
 }
 
+private int QueryInterface(int riid, int ppvObject) {
+	
+	if (riid == 0 || ppvObject == 0)
+		return COM.E_INVALIDARG;
+	GUID guid = new GUID();
+	COM.MoveMemory(guid, riid, GUID.sizeof);
+	if (COM.IsEqualGUID(guid, COM.IIDIUnknown) || COM.IsEqualGUID(guid, COM.IIDIDropTarget)) {
+		COM.MoveMemory(ppvObject, new int[] {iDropTarget.getAddress()}, 4);
+		AddRef();
+		return COM.S_OK;
+	}
+
+	COM.MoveMemory(ppvObject, new int[] {0}, 4);
+	return COM.E_NOINTERFACE;
+}
+
+private int Release() {
+	refCount--;
+	
+	if (refCount == 0) {
+		disposeCOMInterfaces();
+		COM.CoFreeUnusedLibraries();
+	}
+	
+	return refCount;
+}
+
 /**
  * Removes the listener from the collection of listeners who will
  * be notified when a drag and drop operation is in progress.
@@ -642,7 +571,7 @@ private int osOpToOp(int osOperation){
  * @see DropTargetListener
  * @see #addDropListener
  */
-public void removeDropListener(DropTargetListener listener) {
+public void removeDropListener(DropTargetListener listener) {	
 	if (listener == null) DND.error (SWT.ERROR_NULL_ARGUMENT);
 	removeListener (DND.DragEnter, listener);
 	removeListener (DND.DragLeave, listener);
@@ -652,6 +581,82 @@ public void removeDropListener(DropTargetListener listener) {
 	removeListener (DND.DropAccept, listener);
 }
 
+private boolean setEventData(DNDEvent event, int pDataObject, int grfKeyState, int pt_x, int pt_y, int pdwEffect) {	
+	if (pDataObject == 0 || pdwEffect == 0) return false;
+	
+	// get allowed operations
+	int style = getStyle();
+	int[] operations = new int[1];
+	OS.MoveMemory(operations, pdwEffect, 4);
+	operations[0] = osToOp(operations[0]) & style;
+	if (operations[0] == DND.DROP_NONE) return false;
+	
+	// get current operation
+	int operation = getOperationFromKeyState(grfKeyState);
+	keyOperation = operation;
+	if (operation == DND.DROP_DEFAULT) {
+		if ((style & DND.DROP_DEFAULT) == 0) {
+			operation = (operations[0] & DND.DROP_MOVE) != 0 ? DND.DROP_MOVE : DND.DROP_NONE;
+		}
+	} else {
+		if ((operation & operations[0]) == 0) operation = DND.DROP_NONE;
+	}
+	
+	// Get allowed transfer types
+	TransferData[] dataTypes = new TransferData[0];
+	IDataObject dataObject = new IDataObject(pDataObject);
+	dataObject.AddRef();
+	try {
+		int[] address = new int[1];
+		if (dataObject.EnumFormatEtc(COM.DATADIR_GET, address) != COM.S_OK) {
+			return false;
+		}
+		IEnumFORMATETC enumFormatetc = new IEnumFORMATETC(address[0]);
+		try {
+			// Loop over enumerator and save any types that match what we are looking for
+			int rgelt = OS.GlobalAlloc(OS.GMEM_FIXED | OS.GMEM_ZEROINIT, FORMATETC.sizeof);
+			try {
+				int[] pceltFetched = new int[1];
+				enumFormatetc.Reset();
+				while (enumFormatetc.Next(1, rgelt, pceltFetched) == COM.S_OK && pceltFetched[0] == 1) {
+					TransferData transferData = new TransferData();
+					transferData.formatetc = new FORMATETC();
+					COM.MoveMemory(transferData.formatetc, rgelt, FORMATETC.sizeof);
+					transferData.type = transferData.formatetc.cfFormat;
+					transferData.pIDataObject = pDataObject;
+					for (int i = 0; i < transferAgents.length; i++){
+						if (transferAgents[i].isSupportedType(transferData)){
+							TransferData[] newDataTypes = new TransferData[dataTypes.length + 1];
+							System.arraycopy(dataTypes, 0, newDataTypes, 0, dataTypes.length);
+							newDataTypes[dataTypes.length] = transferData;
+							dataTypes = newDataTypes;
+							break;
+						}
+					}
+				}
+			} finally {
+				OS.GlobalFree(rgelt);
+			}
+		} finally {
+			enumFormatetc.Release();
+		}
+	} finally {
+		dataObject.Release();
+	}
+	if (dataTypes.length == 0) return false;
+	
+	event.widget = this;
+	event.x = pt_x;
+	event.y = pt_y;
+	event.time = OS.GetMessageTime();
+	event.feedback = DND.FEEDBACK_SELECT;
+	event.dataTypes = dataTypes;
+	event.dataType = dataTypes[0];
+	event.operations = operations[0];
+	event.detail = operation;
+	return true;
+}
+
 /**
  * Specifies the data types that can be transferred to this DropTarget.  If data is 
  * being dragged that does not match one of these types, the drop target will be notified of 
@@ -667,114 +672,7 @@ public void removeDropListener(DropTargetListener listener) {
  */
 public void setTransfer(Transfer[] transferAgents){
 	if (transferAgents == null) DND.error(SWT.ERROR_NULL_ARGUMENT);
-	
-	if (this.transferAgents.length != 0) {
-		OS.gtk_drag_dest_unset(control.handle);
-	}
 	this.transferAgents = transferAgents;
-	
-	GtkTargetEntry[] targets = new GtkTargetEntry[0];
-	for (int i = 0; i < transferAgents.length; i++) {
-		Transfer transfer = transferAgents[i];
-		int[] typeIds = transfer.getTypeIds();
-		String[] typeNames = transfer.getTypeNames();
-		for (int j = 0; j < typeIds.length; j++) {
-			GtkTargetEntry entry = new GtkTargetEntry();
-			byte[] buffer = Converter.wcsToMbcs(null, typeNames[j], true);
-			entry.target = OS.g_malloc(buffer.length);
-			OS.memmove(entry.target, buffer, buffer.length);						
-			entry.info = typeIds[j];
-			GtkTargetEntry[] newTargets = new GtkTargetEntry[targets.length + 1];
-			System.arraycopy(targets, 0, newTargets, 0, targets.length);
-			newTargets[targets.length] = entry;
-			targets = newTargets;
-		}	
-	}
-	
-	int /*long*/ pTargets = OS.g_malloc(targets.length * GtkTargetEntry.sizeof);
-	for (int i = 0; i < targets.length; i++) {
-		OS.memmove(pTargets + i*GtkTargetEntry.sizeof, targets[i], GtkTargetEntry.sizeof);		
-	}			
-	
-	int actions = opToOsOp(getStyle());
-	OS.gtk_drag_dest_set(control.handle, 0, pTargets, targets.length, actions);
-	
-	for (int i = 0; i < targets.length; i++) {
-		OS.g_free(targets[i].target);
-	}
-}
-
-private boolean setEventData(int /*long*/ context, int x, int y, int time, DNDEvent event) {
-	if (context == 0) return false;
-	GdkDragContext dragContext = new GdkDragContext();
-	OS.memmove(dragContext, context, GdkDragContext.sizeof);
-	if (dragContext.targets == 0) return false;
-	
-	// get allowed operations
-	int style = getStyle();
-	int operations = osOpToOp(dragContext.actions) & style;
-	if (operations == DND.DROP_NONE) return false;
-	
-	// get current operation
-	int operation = getOperationFromKeyState();
-	keyOperation = operation;
-	if (operation == DND.DROP_DEFAULT) {
-		if ((style & DND.DROP_DEFAULT) == 0) {
-			operation = (operations & DND.DROP_MOVE) != 0 ? DND.DROP_MOVE : DND.DROP_NONE;
-		}
-	} else {
-		if ((operation & operations) == 0) operation = DND.DROP_NONE;
-	}
-
-	// Get allowed transfer types
-	int length = OS.g_list_length(dragContext.targets);
-	TransferData[] dataTypes = new TransferData[0];
-	for (int i = 0; i < length; i++) {
-		int /*long*/ pData = OS.g_list_nth(dragContext.targets, i);
-		GtkTargetPair gtkTargetPair = new GtkTargetPair();
-		OS.memmove(gtkTargetPair, pData, GtkTargetPair.sizeof);
-		TransferData data = new TransferData();
-		data.type = gtkTargetPair.target;
-		for (int j = 0; j < transferAgents.length; j++) {
-			if (transferAgents[j].isSupportedType(data)) {
-				TransferData[] newDataTypes = new TransferData[dataTypes.length + 1];
-				System.arraycopy(dataTypes, 0, newDataTypes, 0, dataTypes.length);
-				newDataTypes[dataTypes.length] = data;
-				dataTypes = newDataTypes;	
-				break;
-			}
-		}
-	}
-	if (dataTypes.length == 0) return false;
-
-	Point coordinates = control.toDisplay( new Point( x, y) );
-	
-	event.widget = this;
-	event.x = coordinates.x;
-	event.y = coordinates.y;
-	event.time = time;
-	event.feedback = DND.FEEDBACK_SELECT;
-	event.dataTypes = dataTypes;
-	event.dataType = dataTypes[0];
-	event.operations = operations;
-	event.detail = operation;
-	return true;
-}
-
-private void updateDragOverHover(long delay, DNDEvent event) {
-	if (delay == 0) {
-		dragOverStart = 0;
-		dragOverEvent = null;
-		return;
-	}
-	dragOverStart = System.currentTimeMillis() + delay;
-	if (dragOverEvent == null) dragOverEvent = new DNDEvent();
-	dragOverEvent.x = event.x;
-	dragOverEvent.y = event.y;
-	TransferData[] dataTypes = new TransferData[ event.dataTypes.length];
-	System.arraycopy( event.dataTypes, 0, dataTypes, 0, dataTypes.length);
-	dragOverEvent.dataTypes  = dataTypes;
-	dragOverEvent.operations = event.operations;
 }
 
 }
