@@ -66,10 +66,12 @@ final public class OleFrame extends Composite
  * @param parent a composite widget (cannot be null)
  * @param style the bitwise OR'ing of widget styles
  *
- * @exception SWTError
- * <ul><li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
+ * @exception IllegalArgumentException <ul>
  *     <li>ERROR_NULL_ARGUMENT when the parent is null
- *     <li>ERROR_INTERFACES_NOT_INITIALIZED when unable to create callbacks for OLE Interfaces</ul>
+ * </ul>
+ * @exception SWTException <ul>
+ *     <li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
+ * </ul>
  *
  */
 public OleFrame(Composite parent, int style) {
@@ -121,7 +123,7 @@ private static void initCheckFocus (final Display display) {
 				// ignore popup menus and dialogs
 				int hwnd = OS.GetFocus();
 				while (hwnd != 0) {
-					int ownerHwnd = OS.GetWindow(hwnd, 4/*GW_OWNER*/);
+					int ownerHwnd = OS.GetWindow(hwnd, OS.GW_OWNER);
 					if (ownerHwnd != 0) {			
 						display.timerExec(time, timer[0]);
 						return;
@@ -155,6 +157,7 @@ private static void initMsgHook(Display display) {
 	if (display.getData(HHOOK) != null) return;
 	final Callback callback = new Callback(OleFrame.class, "getMsgProc", 3); //$NON-NLS-1$
 	int address = callback.getAddress();
+	if (address == 0) SWT.error(SWT.ERROR_NO_MORE_CALLBACKS);
 	int threadId = OS.GetCurrentThreadId();
 	final int hHook = OS.SetWindowsHookEx(OS.WH_GETMESSAGE, address, 0, threadId);
 	if (hHook == 0) {
@@ -214,10 +217,10 @@ static int getMsgProc(int code, int wParam, int lParam) {
 							return 0;
 						}
 					}
+					}
 				}
 			}
 		}
-	}
 	return OS.CallNextHookEx(hHook.intValue(), code, wParam, lParam);
 }
 /**
