@@ -45,6 +45,12 @@ public abstract class Control extends Widget implements Drawable {
 	/**
 	 * the handle to the OS resource 
 	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
 	 */
 	public int handle;
 	Composite parent;
@@ -574,6 +580,10 @@ void fixFocus (Control focusControl) {
 	OS.ClearKeyboardFocus (window);
 }
 
+int focusHandle () {
+	return handle;
+}
+
 /**
  * Forces the receiver to have the <em>keyboard focus</em>, causing
  * all keyboard events to be delivered to it.
@@ -589,6 +599,7 @@ void fixFocus (Control focusControl) {
  */
 public boolean forceFocus () {
 	checkWidget();
+	if (display.focusEvent == SWT.FocusOut) return false;
 	Decorations shell = menuShell ();
 	shell.setSavedFocus (this);
 	if (!isEnabled () || !isVisible ()/* || !isActive ()*/) return false;
@@ -596,8 +607,9 @@ public boolean forceFocus () {
 	shell.setSavedFocus (null);
 	shell.bringToTop (false);
 	if (isDisposed ()) return false;
-	int window = OS.GetControlOwner (handle);
-	OS.SetKeyboardFocus (window, handle, (short) OS.kControlFocusNextPart);
+	int focusHandle = focusHandle ();
+	int window = OS.GetControlOwner (focusHandle);
+	OS.SetKeyboardFocus (window, focusHandle, (short) OS.kControlFocusNextPart);
 	if (isDisposed ()) return false;
 	shell.setSavedFocus (this);
 	return hasFocus ();
@@ -1367,21 +1379,11 @@ int kEventControlSetCursor (int nextHandler, int theEvent, int userData) {
 }
 
 int kEventControlSetFocusPart (int nextHandler, int theEvent, int userData) {
+	display.focusCombo = null;
 	if (!display.ignoreFocus) {
 		short [] part = new short [1];
 		OS.GetEventParameter (theEvent, OS.kEventParamControlPart, OS.typeControlPartCode, null, 2, null, part);
-
-		/*
-		* Feature in the Macintosh.  GetKeyboardFocus() returns NULL during
-		* kEventControlSetFocusPart if the focus part is not kControlFocusNoPart.
-		* The fix is to remember the focus control and return it during
-		* kEventControlSetFocusPart.
-		*/
-		Display display = this.display;
-		if (part [0] != OS.kControlFocusNoPart) display.focusControl = this;
-		sendFocusEvent (part [0] != OS.kControlFocusNoPart, false);
-		if (part [0] != OS.kControlFocusNoPart) display.focusControl = null;
-
+		sendFocusEvent (part [0] != OS.kControlFocusNoPart ? SWT.FocusIn : SWT.FocusOut, false);
 		// widget could be disposed at this point
 		if (isDisposed ()) return OS.noErr;
 	}
@@ -1399,8 +1401,8 @@ int kEventMouseDown (int nextHandler, int theEvent, int userData) {
 	OS.GetEventParameter (theEvent, OS.kEventParamMouseButton, OS.typeMouseButton, null, 2, null, button);
 	int [] clickCount = new int [1];
 	OS.GetEventParameter (theEvent, OS.kEventParamClickCount, OS.typeUInt32, null, 4, null, clickCount);
-	sendMouseEvent (SWT.MouseDown, button [0], theEvent);
-	if (clickCount [0] == 2) sendMouseEvent (SWT.MouseDoubleClick, button [0], theEvent);
+	sendMouseEvent (SWT.MouseDown, button [0], 0, 0, false, theEvent);
+	if (clickCount [0] == 2) sendMouseEvent (SWT.MouseDoubleClick, button [0], 0, 0, false, theEvent);
 	if ((state & GRAB) != 0) display.grabControl = this;
 	/*
 	* It is possible that the shell may be
@@ -1423,71 +1425,55 @@ int kEventMouseDown (int nextHandler, int theEvent, int userData) {
 
 int kEventMouseDragged (int nextHandler, int theEvent, int userData) {
 	if ((state & CANVAS) == 0) {
-		if (isEnabledModal ()) sendMouseEvent (SWT.MouseMove, (short) 0, theEvent);
+		if (isEnabledModal ()) sendMouseEvent (SWT.MouseMove, (short) 0, 0, 0, false, theEvent);
 		display.dragDetect (this);
 	}
 	return OS.eventNotHandledErr;
 }
 
 int kEventMouseMoved (int nextHandler, int theEvent, int userData) {
-	if (isEnabledModal ()) sendMouseEvent (SWT.MouseMove, (short) 0, theEvent);
+	if (isEnabledModal ()) sendMouseEvent (SWT.MouseMove, (short) 0, 0, 0, false, theEvent);
 	return OS.eventNotHandledErr;
 }
 
 int kEventMouseUp (int nextHandler, int theEvent, int userData) {
 	short [] button = new short [1];
 	OS.GetEventParameter (theEvent, OS.kEventParamMouseButton, OS.typeMouseButton, null, 2, null, button);
-	sendMouseEvent (SWT.MouseUp, button [0], theEvent);
+	sendMouseEvent (SWT.MouseUp, button [0], 0, 0, false, theEvent);
 	return OS.eventNotHandledErr;
 }
 
-int kEventRawKey (int nextHandler, int theEvent, int userData) {
-	return OS.eventNotHandledErr;
-}
-
-int kEventRawKeyDown (int nextHandler, int theEvent, int userData) {
-	return kEventRawKey (nextHandler, theEvent, userData);
-}
-
-int kEventRawKeyModifiersChanged (int nextHandler, int theEvent, int userData) {
-	Display display = this.display;
-	int [] modifiers = new int [1];
-	OS.GetEventParameter (theEvent, OS.kEventParamKeyModifiers, OS.typeUInt32, null, modifiers.length * 4, null, modifiers);
-	int lastModifiers = display.lastModifiers;
-	int chord = OS.GetCurrentEventButtonState ();
-	int type = SWT.KeyUp;
-	if ((modifiers [0] & OS.alphaLock) != 0 && (lastModifiers & OS.alphaLock) == 0) type = SWT.KeyDown;
-	if ((modifiers [0] & OS.shiftKey) != 0 && (lastModifiers & OS.shiftKey) == 0) type = SWT.KeyDown;
-	if ((modifiers [0] & OS.controlKey) != 0 && (lastModifiers & OS.controlKey) == 0) type = SWT.KeyDown;
-	if ((modifiers [0] & OS.cmdKey) != 0 && (lastModifiers & OS.cmdKey) == 0) type = SWT.KeyDown;
-	if ((modifiers [0] & OS.optionKey) != 0 && (lastModifiers & OS.optionKey) == 0) type = SWT.KeyDown;
-	if (type == SWT.KeyUp && (modifiers [0] & OS.alphaLock) == 0 && (lastModifiers & OS.alphaLock) != 0) {
-		Event event = new Event ();
-		event.keyCode = SWT.CAPS_LOCK;
-		setInputState (event, SWT.KeyDown, chord, modifiers [0]);
-		sendKeyEvent (SWT.KeyDown, event);
+int kEventMouseWheelMoved (int nextHandler, int theEvent, int userData) {
+	if ((state & IGNORE_WHEEL) != 0) return OS.eventNotHandledErr;
+	short [] wheelAxis = new short [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamMouseWheelAxis, OS.typeMouseWheelAxis, null, 2, null, wheelAxis);
+	int [] wheelDelta = new int [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamMouseWheelDelta, OS.typeSInt32, null, 4, null, wheelDelta);
+	Shell shell = getShell ();
+	Control control = this;
+	while (control != null) {
+		if (!control.sendMouseEvent (SWT.MouseWheel, (short) 0, wheelDelta [0], SWT.SCROLL_LINE, true, theEvent)) {
+			break;
+		}
+		if (control.sendMouseWheel (wheelAxis [0], wheelDelta [0])) {
+			break;
+		}
+		if (control == this) {
+			/*
+			* Feature in the Macintosh.  For some reason, the kEventMouseWheelMoved
+			* event is sent twice to each application handler with the same mouse wheel
+			* data.  The fix is to set an ignore flag before calling the next handler 
+			* in the handler chain.
+			*/
+			state |= IGNORE_WHEEL;
+			int result = OS.CallNextEventHandler(nextHandler, theEvent);
+			state &= ~IGNORE_WHEEL;
+			if (result == OS.noErr) break;
+		}
+		if (control == shell) break;
+		control = control.parent;
 	}
-	Event event = new Event ();
-	setInputState (event, type, chord, modifiers [0]);
-	if (event.keyCode == 0 && event.character == 0) return OS.eventNotHandledErr;
-	boolean result = sendKeyEvent (type, event);
-	if (type == SWT.KeyDown && (modifiers [0] & OS.alphaLock) != 0 && (lastModifiers & OS.alphaLock) == 0) {
-		event = new Event ();
-		event.keyCode = SWT.CAPS_LOCK;
-		setInputState (event, SWT.KeyUp, chord, modifiers [0]);
-		sendKeyEvent (SWT.KeyUp, event);
-	}
-	display.lastModifiers = modifiers [0];
-	return result ? OS.eventNotHandledErr : OS.noErr;
-}
-
-int kEventRawKeyRepeat (int nextHandler, int theEvent, int userData) {
-	return kEventRawKey (nextHandler, theEvent, userData);
-}
-
-int kEventRawKeyUp (int nextHandler, int theEvent, int userData) {
-	if (!sendKeyEvent (SWT.KeyUp, theEvent)) return OS.noErr;
-	return OS.eventNotHandledErr;
+	return OS.noErr;
 }
 
 int kEventTextInputUnicodeForKeyEvent (int nextHandler, int theEvent, int userData) {
@@ -1510,6 +1496,10 @@ int kEventTextInputUnicodeForKeyEvent (int nextHandler, int theEvent, int userDa
 	return OS.eventNotHandledErr;
 }
 
+void markLayout (boolean changed, boolean all) {
+	/* Do nothing */
+}
+
 /**
  * Moves the receiver above the specified control in the
  * drawing order. If the argument is null, then the receiver
@@ -1672,8 +1662,8 @@ void register () {
 }
 
 void releaseChild () {
-	super.releaseChild ();
 	setVisible (topHandle (), false);
+	parent.removeControl (this);
 }
 
 void releaseHandle () {
@@ -1934,14 +1924,22 @@ void resetVisibleRegion (int control) {
 	}
 }
 
-void sendFocusEvent (boolean focusIn, boolean post) {
+void sendFocusEvent (int type, boolean post) {
+	Display display = this.display;
 	Shell shell = getShell ();
+	/*
+	* Feature in the Macintosh.  GetKeyboardFocus() returns NULL during
+	* kEventControlSetFocusPart if the focus part is not kControlFocusNoPart.
+	* The fix is to remember the focus control and return it during
+	* kEventControlSetFocusPart.
+	*/
+	display.focusControl = this;
+	display.focusEvent = type;
 	if (post) {
-		postEvent (focusIn ? SWT.FocusIn : SWT.FocusOut);
+		postEvent (type);
 	} else {
-		sendEvent (focusIn ? SWT.FocusIn : SWT.FocusOut);
-	}
-	
+		sendEvent (type);
+	}	
 	/*
 	* It is possible that the shell may be
 	* disposed at this point.  If this happens
@@ -1949,62 +1947,23 @@ void sendFocusEvent (boolean focusIn, boolean post) {
 	* events.
 	*/
 	if (!shell.isDisposed ()) {
-		if (focusIn) {
-			shell.setActiveControl (this);
-		} else {
-			Display display = shell.display;
-			Control control = display.getFocusControl ();
-			if (control == null || shell != control.getShell () ) {
-				shell.setActiveControl (null);
-			}
+		switch (type) {
+			case SWT.FocusIn:
+				shell.setActiveControl (this);
+				break;
+			case SWT.FocusOut:
+				if (shell != display.getActiveShell ()) {
+					shell.setActiveControl (null);
+				}
+				break;
 		}
 	}
+	display.focusEvent = SWT.None;
+	display.focusControl = null;
 }
 
-boolean sendKeyEvent (int type, int theEvent) {
-	int [] length = new int [1];
-	int status = OS.GetEventParameter (theEvent, OS.kEventParamKeyUnicodes, OS.typeUnicodeText, null, 4, length, (char[])null);
-	if (status == OS.noErr && length [0] > 2) {
-		int count = 0;
-		int [] chord = new int [1];
-		OS.GetEventParameter (theEvent, OS.kEventParamMouseChord, OS.typeUInt32, null, 4, null, chord);
-		int [] modifiers = new int [1];
-		OS.GetEventParameter (theEvent, OS.kEventParamKeyModifiers, OS.typeUInt32, null, 4, null, modifiers);
-		char [] chars = new char [length [0] / 2];
-		OS.GetEventParameter (theEvent, OS.kEventParamKeyUnicodes, OS.typeUnicodeText, null, chars.length * 2, null, chars);
-		for (int i=0; i<chars.length; i++) {
-			Event event = new Event ();
-			event.character = chars [i];
-			setInputState (event, type, chord [0], modifiers [0]);
-			if (sendKeyEvent (type, event)) chars [count++] = chars [i];
-		}
-		if (count == 0) return false;
-		if (count != chars.length - 1) {
-			OS.SetEventParameter (theEvent, OS.kEventParamKeyUnicodes, OS.typeUnicodeText, count * 2, chars);
-		}
-		return true;
-	} else {
-		Event event = new Event ();
-		if (!setKeyState (event, type, theEvent)) return true;
-		return sendKeyEvent (type, event);
-	}
-}
-
-boolean sendKeyEvent (int type, Event event) {
-	sendEvent (type, event);
-	// widget could be disposed at this point
-	
-	/*
-	* It is possible (but unlikely), that application
-	* code could have disposed the widget in the key
-	* events.  If this happens, end the processing of
-	* the key by returning false.
-	*/
-	if (isDisposed ()) return false;
-	return event.doit;
-}
-
-boolean sendMouseEvent (int type, short button, int theEvent) {
+boolean sendMouseEvent (int type, short button, int count, int detail, boolean send, int theEvent) {
+	if (!hooks (type) && !filters (type)) return true;
 	int sizeof = org.eclipse.swt.internal.carbon.Point.sizeof;
 	org.eclipse.swt.internal.carbon.Point pt = new org.eclipse.swt.internal.carbon.Point ();
 	OS.GetEventParameter (theEvent, OS.kEventParamMouseLocation, OS.typeQDPoint, null, sizeof, null, pt);
@@ -2020,21 +1979,34 @@ boolean sendMouseEvent (int type, short button, int theEvent) {
 	OS.GetEventParameter (theEvent, OS.kEventParamMouseChord, OS.typeUInt32, null, 4, null, chord);
 	int [] modifiers = new int [1];
 	OS.GetEventParameter (theEvent, OS.kEventParamKeyModifiers, OS.typeUInt32, null, 4, null, modifiers);
-	return sendMouseEvent (type, button, chord [0], (short) x, (short) y, modifiers [0], false);
+	return sendMouseEvent (type, button, count, detail, chord [0], (short) x, (short) y, modifiers [0], send);
 }
 
 boolean sendMouseEvent (int type, short button, int chord, short x, short y, int modifiers, boolean send) {
+	if (!hooks (type) && !filters (type)) return true;
+	return sendMouseEvent (type, button, 0, 0, chord, x, y, modifiers, send);
+}
+
+boolean sendMouseEvent (int type, short button, int count, int detail, int chord, short x, short y, int modifiers, boolean send) {
 	Event event = new Event ();
 	switch (button) {
 		case 1: event.button = 1; break;
 		case 2: event.button = 3; break;
 		case 3: event.button = 2; break;
+		case 4: event.button = 4; break;
+		case 5: event.button = 5; break;
 	}
 	event.x = x;
 	event.y = y;
+	event.count = count;
+	event.detail = detail;
 	setInputState (event, type, chord, modifiers);
 	sendEvent (type, event, send);
-	return true;
+	return event.doit;
+}
+
+boolean sendMouseWheel (short wheelAxis, int wheelDelta) {
+	return false;
 }
 
 /**
@@ -2101,7 +2073,11 @@ void setBackground (float [] color) {
  */
 public void setBounds (int x, int y, int width, int height) {
 	checkWidget();
-	setBounds (topHandle (), x, y, width, height, true, true, true);
+	setBounds (x, y, Math.max (0, width), Math.max (0, height), true, true, true);
+}
+
+int setBounds (int x, int y, int width, int height, boolean move, boolean resize, boolean events) {
+	return setBounds (topHandle (), x, y, width, height, move, resize, events);
 }
 
 /**
@@ -2244,8 +2220,10 @@ public void setEnabled (boolean enabled) {
 	Control control = null;
 	boolean fixFocus = false;
 	if (!enabled) {
-		control = display.getFocusControl ();
-		fixFocus = isFocusAncestor (control);
+		if (display.focusEvent != SWT.FocusOut) {
+			control = display.getFocusControl ();
+			fixFocus = isFocusAncestor (control);
+		}
 	}
 	if (enabled) {
 		state &= ~DISABLED;
@@ -2387,7 +2365,7 @@ public void setLayoutData (Object layoutData) {
  */
 public void setLocation (int x, int y) {
 	checkWidget();
-	setBounds (topHandle (), x, y, 0, 0, true, false, true);
+	setBounds (x, y, 0, 0, true, false, true);
 }
 
 /**
@@ -2524,7 +2502,7 @@ boolean setRadioSelection (boolean value){
  */
 public void setSize (int width, int height) {
 	checkWidget();
-	setBounds (topHandle (), 0, 0, width, height, false, true, true);
+	setBounds (0, 0, Math.max (0, width), Math.max (0, height), false, true, true);
 }
 
 /**
@@ -2621,8 +2599,10 @@ public void setVisible (boolean visible) {
 	Control control = null;
 	boolean fixFocus = false;
 	if (!visible) {
-		control = display.getFocusControl ();
-		fixFocus = isFocusAncestor (control);
+		if (display.focusEvent != SWT.FocusOut) {
+			control = display.getFocusControl ();
+			fixFocus = isFocusAncestor (control);
+		}
 	}
 	setVisible (topHandle (), visible);
 	if (!visible) {
@@ -3017,4 +2997,8 @@ void update (boolean all) {
 	OS.DisposeRgn (portRgn);
 }
 
+void updateLayout (boolean all) {
+	/* Do nothing */
+}
+
 }
