@@ -12,9 +12,10 @@ package org.eclipse.swt.widgets;
 
 
 import org.eclipse.swt.*;
+import org.eclipse.swt.internal.*;
+import org.eclipse.swt.internal.gtk.*;
 import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.events.*;
-import org.eclipse.swt.internal.carbon.*;
 
 /**
  * Instances of this class represent a selectable user interface object
@@ -34,8 +35,9 @@ import org.eclipse.swt.internal.carbon.*;
  */
 public class MenuItem extends Item {
 	Menu parent, menu;
+	int /*long*/ groupHandle;
 	int accelerator;
-
+	
 /**
  * Constructs a new instance of this class given its parent
  * (which must be a <code>Menu</code>) and a style value
@@ -73,7 +75,7 @@ public class MenuItem extends Item {
 public MenuItem (Menu parent, int style) {
 	super (parent, checkStyle (style));
 	this.parent = parent;
-	parent.createItem (this, parent.getItemCount ());
+	createWidget (parent.getItemCount ());
 }
 
 /**
@@ -114,22 +116,20 @@ public MenuItem (Menu parent, int style) {
 public MenuItem (Menu parent, int style, int index) {
 	super (parent, checkStyle (style));
 	this.parent = parent;
-	parent.createItem (this, index);
+	int count = parent.getItemCount ();
+	if (!(0 <= index && index <= count)) {
+		error (SWT.ERROR_INVALID_RANGE);
+	}
+	createWidget (index);
 }
 
-public void _setEnabled (boolean enabled) {
-	int index = parent.indexOf (this);
-	if (index == -1) return;
-	int outMenuRef [] = new int [1];
-	short menuIndex = (short) (index + 1);
-	OS.GetMenuItemHierarchicalMenu (parent.handle, menuIndex, outMenuRef);
-	if (enabled) {
-		if (outMenuRef [0] != 0) OS.EnableMenuItem (outMenuRef [0], (short) 0);
-		OS.EnableMenuItem (parent.handle, menuIndex);
-	} else {
-		if (outMenuRef [0] != 0) OS.DisableMenuItem (outMenuRef [0], (short) 0);
-		OS.DisableMenuItem (parent.handle, menuIndex);
-	}
+void addAccelerator (int /*long*/ accelGroup) {
+	updateAccelerator (accelGroup, true);
+}
+
+void addAccelerators (int /*long*/ accelGroup) {
+	addAccelerator (accelGroup);
+	if (menu != null) menu.addAccelerators (accelGroup);
 }
 
 /**
@@ -152,7 +152,7 @@ public void _setEnabled (boolean enabled) {
  * @see #removeArmListener
  */
 public void addArmListener (ArmListener listener) {
-	checkWidget ();
+	checkWidget();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	TypedListener typedListener = new TypedListener (listener);
 	addListener (SWT.Arm, typedListener);
@@ -178,7 +178,7 @@ public void addArmListener (ArmListener listener) {
  * @see #removeHelpListener
  */
 public void addHelpListener (HelpListener listener) {
-	checkWidget ();
+	checkWidget();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	TypedListener typedListener = new TypedListener (listener);
 	addListener (SWT.Help, typedListener);
@@ -209,19 +209,71 @@ public void addHelpListener (HelpListener listener) {
  * @see SelectionEvent
  */
 public void addSelectionListener (SelectionListener listener) {
-	checkWidget ();
+	checkWidget();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	TypedListener typedListener = new TypedListener(listener);
 	addListener (SWT.Selection,typedListener);
 	addListener (SWT.DefaultSelection,typedListener);
 }
 
+static int checkStyle (int style) {
+	return checkBits (style, SWT.PUSH, SWT.CHECK, SWT.RADIO, SWT.SEPARATOR, SWT.CASCADE, 0);
+}
+
 protected void checkSubclass () {
 	if (!isValidSubclass ()) error (SWT.ERROR_INVALID_SUBCLASS);
 }
 
-static int checkStyle (int style) {
-	return checkBits (style, SWT.PUSH, SWT.CHECK, SWT.RADIO, SWT.SEPARATOR, SWT.CASCADE, 0);
+void createHandle (int index) {
+	state |= HANDLE;
+	byte [] buffer = new byte [1];
+	int bits = SWT.CHECK | SWT.RADIO | SWT.PUSH | SWT.SEPARATOR;
+	switch (style & bits) {
+		case SWT.SEPARATOR:
+			handle = OS.gtk_separator_menu_item_new ();
+			break;
+		case SWT.RADIO:
+			/*
+			* Feature in GTK.  In GTK, radio button must always be part of
+			* a radio button group.  In a GTK radio group, one button is always
+			* selected.  This means that it is not possible to have a single
+			* radio button that is unselected.  This is necessary to allow
+			* applications to implement their own radio behavior or use radio
+			* buttons outside of radio groups.  The fix is to create a hidden
+			* radio button for each radio button we create and add them
+			* to the same group.  This allows the visible button to be
+			* unselected.
+			*/
+			groupHandle = OS.gtk_radio_menu_item_new (0);
+			if (groupHandle == 0) error (SWT.ERROR_NO_HANDLES);
+			OS.g_object_ref (groupHandle);
+			OS.gtk_object_sink (groupHandle);
+			int /*long*/ group = OS.gtk_radio_menu_item_get_group (groupHandle);
+			handle = OS.gtk_radio_menu_item_new_with_label (group, buffer);
+			break;
+		case SWT.CHECK:
+			handle = OS.gtk_check_menu_item_new_with_label (buffer);
+			break;
+		case SWT.PUSH:
+		default:
+			handle = OS.gtk_image_menu_item_new_with_label (buffer);
+			break;
+	}
+	if (handle == 0) error (SWT.ERROR_NO_HANDLES);
+	if ((style & SWT.SEPARATOR) == 0) {
+		int /*long*/ label = OS.gtk_bin_get_child (handle);
+		OS.gtk_accel_label_set_accel_widget (label, 0);
+	}
+	int /*long*/ parentHandle = parent.handle;
+	boolean enabled = OS.GTK_WIDGET_SENSITIVE (parentHandle);     
+	if (!enabled) OS.GTK_WIDGET_SET_FLAGS (parentHandle, OS.GTK_SENSITIVE);
+	OS.gtk_menu_shell_insert (parentHandle, handle, index);
+	if (!enabled) OS.GTK_WIDGET_UNSET_FLAGS (parentHandle, OS.GTK_SENSITIVE);
+	OS.gtk_widget_show (handle);
+}
+
+void fixMenus (Decorations newParent) {
+	if (menu != null) menu.fixMenus (newParent);
 }
 
 /**
@@ -238,10 +290,20 @@ static int checkStyle (int style) {
  * </ul>
  */
 public int getAccelerator () {
-	checkWidget ();
+	checkWidget();
 	return accelerator;
 }
 
+int /*long*/ getAccelGroup () {
+	Menu menu = parent;
+	while (menu != null && menu.cascade != null) {
+		menu = menu.cascade.parent;
+	}
+	if (menu == null) return 0;
+	Decorations shell = menu.parent;
+	return shell.menuBar == menu ? shell.accelGroup : 0;
+}
+
 /**
  * Returns <code>true</code> if the receiver is enabled, and
  * <code>false</code> otherwise. A disabled control is typically
@@ -259,7 +321,7 @@ public int getAccelerator () {
  */
 public boolean getEnabled () {
 	checkWidget();
-	return (state & DISABLED) == 0;
+	return OS.GTK_WIDGET_SENSITIVE (handle);
 }
 
 /**
@@ -277,15 +339,10 @@ public boolean getEnabled () {
  * </ul>
  */
 public Menu getMenu () {
-	checkWidget ();
+	checkWidget();
 	return menu;
 }
 
-String getNameText () {
-	if ((style & SWT.SEPARATOR) != 0) return "|";
-	return super.getNameText ();
-}
-
 /**
  * Returns the receiver's parent, which must be a <code>Menu</code>.
  *
@@ -297,7 +354,7 @@ String getNameText () {
  * </ul>
  */
 public Menu getParent () {
-	checkWidget ();
+	checkWidget();
 	return parent;
 }
 
@@ -316,36 +373,76 @@ public Menu getParent () {
  * </ul>
  */
 public boolean getSelection () {
-	checkWidget ();
+	checkWidget();
 	if ((style & (SWT.CHECK | SWT.RADIO)) == 0) return false;
-	int index = parent.indexOf (this);
-	if (index == -1) return false;
-	short [] outMark = new short [1];
-	OS.GetItemMark (parent.handle, (short) (index + 1), outMark);
-	return outMark [0] != 0;
+	return OS.gtk_check_menu_item_get_active(handle);
 }
 
-int kEventProcessCommand (int nextHandler, int theEvent, int userData) {
-	//TEMPORARY CODE
-	if (!isEnabled ()) return OS.noErr;
-
-	if ((style & SWT.CHECK) != 0) {
-		setSelection (!getSelection ());
-	} else {
-		if ((style & SWT.RADIO) != 0) {
-			if ((parent.getStyle () & SWT.NO_RADIO_GROUP) != 0) {
-				setSelection (!getSelection ());
-			} else {
-				selectRadio ();
+int /*long*/ gtk_activate (int /*long*/ widget) {
+	if ((style & SWT.CASCADE) != 0 && menu != null) return 0;
+	/*
+	* Bug in GTK.  When an ancestor menu is disabled and
+	* the user types an accelerator key, GTK delivers the
+	* the activate signal even though the menu item cannot
+	* be invoked using the mouse.  The fix is to ignore activate
+	* signals when an ancestor menu is disabled.
+	*/
+	if (!isEnabled ()) return 0;
+	Event event = new Event ();
+	int /*long*/ ptr = OS.gtk_get_current_event ();
+	if (ptr != 0) {
+		GdkEvent gdkEvent = new GdkEvent ();
+		OS.memmove (gdkEvent, ptr, GdkEvent.sizeof);
+		switch (gdkEvent.type) {
+			case OS.GDK_KEY_PRESS:
+			case OS.GDK_KEY_RELEASE: 
+			case OS.GDK_BUTTON_PRESS:
+			case OS.GDK_2BUTTON_PRESS: 
+			case OS.GDK_BUTTON_RELEASE: {
+				int [] state = new int [1];
+				OS.gdk_event_get_state (ptr, state);
+				setInputState (event, state [0]);
+				break;
 			}
 		}
+		OS.gdk_event_free (ptr);
+	}
+	if ((style & SWT.RADIO) != 0) {
+		if ((parent.getStyle () & SWT.NO_RADIO_GROUP) == 0) {
+			selectRadio ();
+		}
 	}
-	int [] modifiers = new int [1];
-	OS.GetEventParameter (theEvent, OS.kEventParamKeyModifiers, OS.typeUInt32, null, 4, null, modifiers);
-	Event event = new Event ();
-	setInputState (event, (short) 0, OS.GetCurrentEventButtonState (), modifiers [0]);
 	postEvent (SWT.Selection, event);
-	return OS.noErr;
+	return 0;
+}
+
+int /*long*/ gtk_select (int /*long*/ item) {
+	parent.selectedItem = this;
+	sendEvent (SWT.Arm);
+	return 0;
+}
+
+int /*long*/ gtk_show_help (int /*long*/ widget, int /*long*/ helpType) {
+	boolean handled = hooks (SWT.Help);
+	if (handled) {
+		postEvent (SWT.Help);
+	} else {
+		handled = parent.sendHelpEvent (helpType);
+	}
+	if (handled) {
+		OS.gtk_menu_shell_deactivate (parent.handle);
+		return 1;
+	}
+	return 0;
+}
+
+void hookEvents () {
+	super.hookEvents ();
+	int /*long*/ windowProc2 = display.windowProc2;
+	int /*long*/ windowProc3 = display.windowProc3;
+	OS.g_signal_connect (handle, OS.activate, windowProc2, ACTIVATE);
+	OS.g_signal_connect (handle, OS.select, windowProc2, SELECT);
+	OS.g_signal_connect (handle, OS.show_help, windowProc3, SHOW_HELP);
 }
 
 /**
@@ -367,73 +464,36 @@ public boolean isEnabled () {
 	return getEnabled () && parent.isEnabled ();
 }
 
-int keyGlyph (int key) {
-	switch (key) {
-		case SWT.BS: return OS.kMenuDeleteLeftGlyph;
-		case SWT.CR: return OS.kMenuReturnGlyph;
-		case SWT.DEL: return OS.kMenuDeleteRightGlyph;
-		case SWT.ESC: return OS.kMenuEscapeGlyph;
-		case SWT.LF: return OS.kMenuReturnGlyph;
-		case SWT.TAB: return OS.kMenuTabRightGlyph;
-		case ' ': return OS.kMenuBlankGlyph;
-//		case ' ': return OS.kMenuSpaceGlyph;
-		case SWT.ALT: return OS.kMenuOptionGlyph;
-		case SWT.SHIFT: return OS.kMenuShiftGlyph;
-		case SWT.CONTROL: return OS.kMenuControlISOGlyph;
-		case SWT.COMMAND: return OS.kMenuCommandGlyph;
-		case SWT.ARROW_UP: return OS.kMenuUpArrowGlyph;
-		case SWT.ARROW_DOWN: return OS.kMenuDownArrowGlyph;
-		case SWT.ARROW_LEFT: return OS.kMenuLeftArrowGlyph;
-		case SWT.ARROW_RIGHT: return OS.kMenuRightArrowGlyph;
-		case SWT.PAGE_UP: return OS.kMenuPageUpGlyph;
-		case SWT.PAGE_DOWN: return OS.kMenuPageDownGlyph;
-		case SWT.F1: return OS.kMenuF1Glyph;
-		case SWT.F2: return OS.kMenuF2Glyph;
-		case SWT.F3: return OS.kMenuF3Glyph;
-		case SWT.F4: return OS.kMenuF4Glyph;
-		case SWT.F5: return OS.kMenuF5Glyph;
-		case SWT.F6: return OS.kMenuF6Glyph;
-		case SWT.F7: return OS.kMenuF7Glyph;
-		case SWT.F8: return OS.kMenuF8Glyph;
-		case SWT.F9: return OS.kMenuF9Glyph;
-		case SWT.F10: return OS.kMenuF10Glyph;
-		case SWT.F11: return OS.kMenuF11Glyph;
-		case SWT.F12: return OS.kMenuF12Glyph;
-	}
-	return OS.kMenuNullGlyph;
-}
-
 void releaseChild () {
 	super.releaseChild ();
-	if (menu != null) menu.dispose ();
+	if (menu != null) {
+		if (menu.selectedItem == this) menu.selectedItem = null;
+		menu.dispose ();
+	}
 	menu = null;
-	parent.destroyItem (this);
 }
 
 void releaseWidget () {
-	if (menu != null) {
-		menu.releaseWidget ();
-		menu.destroyWidget ();
-	} else {
-		if ((parent.style & SWT.BAR) != 0) {
-//			short [] outIndex = new short [1];
-//			if (OS.GetIndMenuItemWithCommandID (parent.handle, id, 1, null, outIndex) == OS.noErr) {
-//				int [] outMenuRef = new int [1];
-//				OS.GetMenuItemHierarchicalMenu (parent.handle, outIndex [0], outMenuRef);
-//				if (outMenuRef [0] != 0) {
-//					OS.DeleteMenu (OS.GetMenuID (outMenuRef [0]));
-//					OS.DisposeMenu (outMenuRef [0]);
-//				}
-//			}
-		}
-	}
+	if (menu != null) menu.releaseResources ();
 	menu = null;
 	super.releaseWidget ();
+	int /*long*/ accelGroup = getAccelGroup ();
+	if (accelGroup != 0) removeAccelerator (accelGroup);
+	if (groupHandle != 0) OS.g_object_unref (groupHandle);
+	groupHandle = 0;
 	accelerator = 0;
-	if (this == parent.defaultItem) parent.defaultItem = null;
 	parent = null;
 }
 
+void removeAccelerator (int /*long*/ accelGroup) {
+	updateAccelerator (accelGroup, false);
+}
+
+void removeAccelerators (int /*long*/ accelGroup) {
+	removeAccelerator (accelGroup);
+	if (menu != null) menu.removeAccelerators (accelGroup);
+}
+
 /**
  * Removes the listener from the collection of listeners who will
  * be notified when the arm events are generated for the control.
@@ -452,7 +512,7 @@ void releaseWidget () {
  * @see #addArmListener
  */
 public void removeArmListener (ArmListener listener) {
-	checkWidget ();
+	checkWidget();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (eventTable == null) return;
 	eventTable.unhook (SWT.Arm, listener);
@@ -476,7 +536,7 @@ public void removeArmListener (ArmListener listener) {
  * @see #addHelpListener
  */
 public void removeHelpListener (HelpListener listener) {
-	checkWidget ();
+	checkWidget();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (eventTable == null) return;
 	eventTable.unhook (SWT.Help, listener);
@@ -500,13 +560,12 @@ public void removeHelpListener (HelpListener listener) {
  * @see #addSelectionListener
  */
 public void removeSelectionListener (SelectionListener listener) {
-	checkWidget ();
+	checkWidget();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (eventTable == null) return;
 	eventTable.unhook (SWT.Selection, listener);
 	eventTable.unhook (SWT.DefaultSelection,listener);	
 }
-
 void selectRadio () {
 	int index = 0;
 	MenuItem [] items = parent.getItems ();
@@ -517,7 +576,6 @@ void selectRadio () {
 	while (j < items.length && items [j].setRadioSelection (false)) j++;
 	setSelection (true);
 }
-
 /**
  * Sets the widget accelerator.  An accelerator is the bit-wise
  * OR of zero or more modifier masks and a key. Examples:
@@ -533,35 +591,12 @@ void selectRadio () {
  * </ul>
  */
 public void setAccelerator (int accelerator) {
-	checkWidget ();
-	int index = parent.indexOf (this);
-	if (index == -1) return;
-	boolean update = (this.accelerator == 0 && accelerator != 0) || (this.accelerator != 0 && accelerator == 0);
+	checkWidget();
+	if (this.accelerator == accelerator) return;
+	int /*long*/ accelGroup = getAccelGroup ();
+	if (accelGroup != 0) removeAccelerator (accelGroup);
 	this.accelerator = accelerator;
-	boolean inSetVirtualKey = false;
-	int inModifiers = OS.kMenuNoModifiers, inGlyph = OS.kMenuNullGlyph, inKey = 0;
-	if (accelerator != 0) {
-		inKey = accelerator & SWT.KEY_MASK;
-		inGlyph = keyGlyph (inKey);
-		int virtualKey = Display.untranslateKey (inKey);
-		if (inKey == ' ') virtualKey = 49;
-		if (virtualKey != 0) {
-			inSetVirtualKey = true;
-			inKey = virtualKey;
-		} else {
-			inKey = Character.toUpperCase ((char)inKey);
-		}
-		inModifiers = (byte) OS.kMenuNoCommandModifier;
-		if ((accelerator & SWT.SHIFT) != 0) inModifiers |= OS.kMenuShiftModifier;
-		if ((accelerator & SWT.CONTROL) != 0) inModifiers |= OS.kMenuControlModifier;
-		if ((accelerator & SWT.COMMAND) != 0) inModifiers &= ~OS.kMenuNoCommandModifier;
-		if ((accelerator & SWT.ALT) != 0) inModifiers |= OS.kMenuOptionModifier;
-	}
-	short menuIndex = (short) (index + 1);
-	OS.SetMenuItemModifiers (parent.handle, menuIndex, (byte)inModifiers);
-	OS.SetMenuItemCommandKey (parent.handle, menuIndex, inSetVirtualKey, (char)inKey);
-	OS.SetMenuItemKeyGlyph (parent.handle, menuIndex, (short)inGlyph);
-	if (update) updateText (menuIndex);
+	if (accelGroup != 0) addAccelerator (accelGroup);
 }
 
 /**
@@ -578,13 +613,12 @@ public void setAccelerator (int accelerator) {
  * </ul>
  */
 public void setEnabled (boolean enabled) {
-	checkWidget ();
-	if (enabled) {
-		state &= ~DISABLED;
-	} else {
-		state |= DISABLED;
-	}
-	_setEnabled (enabled);
+	checkWidget();
+	if (OS.GTK_WIDGET_SENSITIVE (handle) == enabled) return;
+	int /*long*/ accelGroup = getAccelGroup ();
+	if (accelGroup != 0) removeAccelerator (accelGroup);
+	OS.gtk_widget_set_sensitive (handle, enabled);
+	if (accelGroup != 0) addAccelerator (accelGroup);
 }
 
 /**
@@ -601,14 +635,22 @@ public void setEnabled (boolean enabled) {
  * </ul>
  */
 public void setImage (Image image) {
-	checkWidget ();
+	checkWidget();
 	if ((style & SWT.SEPARATOR) != 0) return;
-	int index = parent.indexOf (this);
-	if (index == -1) return;
 	super.setImage (image);
-	int imageHandle = image != null ? image.handle : 0;
-	byte type = image != null ? (byte)OS.kMenuCGImageRefType : (byte)OS.kMenuNoIcon;
-	OS.SetMenuItemIconHandle (parent.handle, (short) (index + 1), type, imageHandle);
+	if (!OS.GTK_IS_IMAGE_MENU_ITEM (handle)) return;
+	if (image != null) {
+		ImageList imageList = parent.imageList;
+		if (imageList == null) imageList = parent.imageList = new ImageList ();
+		int imageIndex = imageList.indexOf (image);
+		if (imageIndex == -1) imageIndex = imageList.add (image);
+		int /*long*/ pixbuf = imageList.getPixbuf (imageIndex);
+		int /*long*/ imageHandle = OS.gtk_image_new_from_pixbuf (pixbuf);
+		OS.gtk_image_menu_item_set_image (handle, imageHandle);
+		OS.gtk_widget_show (imageHandle);
+	} else {
+		OS.gtk_image_menu_item_set_image (handle, 0);
+	}
 }
 
 /**
@@ -639,7 +681,6 @@ public void setMenu (Menu menu) {
 		error (SWT.ERROR_MENUITEM_NOT_CASCADE);
 	}
 	if (menu != null) {
-		if (menu.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
 		if ((menu.style & SWT.DROP_DOWN) == 0) {
 			error (SWT.ERROR_MENU_NOT_DROP_DOWN);
 		}
@@ -647,48 +688,26 @@ public void setMenu (Menu menu) {
 			error (SWT.ERROR_INVALID_PARENT);
 		}
 	}
-	
+
 	/* Assign the new menu */
 	Menu oldMenu = this.menu;
 	if (oldMenu == menu) return;
-	if (oldMenu != null) oldMenu.cascade = null;
-	this.menu = menu;
-	
-	/* Update the menu in the OS */
-	int index = parent.indexOf (this);
-	if (index == -1) return;
-	short menuIndex = (short) (index + 1);
-	int outMenuRef [] = new int [1];
-	if (menu == null) {
-		if ((parent.style & SWT.BAR) != 0) {
-//			Display display = getDisplay ();
-//			short menuID = display.nextMenuId ();
-//			if (OS.CreateNewMenu (menuID, 0, outMenuRef) != OS.noErr) {
-//				error (SWT.ERROR_NO_HANDLES);
-//			}
-		}
-	} else {
+	int /*long*/ accelGroup = getAccelGroup ();
+	if (accelGroup != 0) removeAccelerators (accelGroup);
+	if (oldMenu != null) {
+		oldMenu.cascade = null;
+		/*
+		* Add a reference to the menu we are about
+		* to replace or GTK will destroy it.
+		*/
+		OS.g_object_ref (oldMenu.handle);
+		OS.gtk_menu_item_remove_submenu (handle);
+	}
+	if ((this.menu = menu) != null) {
 		menu.cascade = this;
-		if ((parent.style & SWT.BAR) != 0) {
-			if (oldMenu == null) {
-//				OS.GetMenuItemHierarchicalMenu (parent.handle, outIndex [0], outMenuRef);
-//				if (outMenuRef [0] != 0) {
-//					OS.DeleteMenu (OS.GetMenuID (outMenuRef [0]));
-//					OS.DisposeMenu (outMenuRef [0]);
-//				}
-			}
-		}
-		outMenuRef [0] = menu.handle;
-		int [] outString = new int [1];
-		if (OS.CopyMenuItemTextAsCFString (parent.handle, menuIndex, outString) != OS.noErr) {
-			error (SWT.ERROR_CANNOT_SET_MENU);
-		}
-		OS.SetMenuTitleWithCFString (outMenuRef [0], outString [0]);
-		OS.CFRelease (outString [0]);
-	}
-	if (OS.SetMenuItemHierarchicalMenu (parent.handle, menuIndex, outMenuRef [0]) != OS.noErr) {
-		error (SWT.ERROR_CANNOT_SET_MENU);
+		OS.gtk_menu_item_set_submenu (handle, menu.handle);
 	}
+	if (accelGroup != 0) addAccelerators (accelGroup);
 }
 
 boolean setRadioSelection (boolean value) {
@@ -714,12 +733,12 @@ boolean setRadioSelection (boolean value) {
  * </ul>
  */
 public void setSelection (boolean selected) {
-	checkWidget ();
+	checkWidget();
 	if ((style & (SWT.CHECK | SWT.RADIO)) == 0) return;
-	int index = parent.indexOf (this);
-	if (index == -1) return;
-	int inMark = selected ? ((style & SWT.RADIO) != 0) ? OS.diamondMark : OS.checkMark : 0;
-	OS.SetItemMark (parent.handle, (short) (index + 1), (short) inMark);
+	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, ACTIVATE);
+	OS.gtk_check_menu_item_set_active (handle, selected);
+	if ((style & SWT.RADIO) != 0) OS.gtk_check_menu_item_set_active (groupHandle, !selected);
+	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, ACTIVATE);
 }
 
 /**
@@ -760,35 +779,48 @@ public void setSelection (boolean selected) {
  * @see #setAccelerator
  */
 public void setText (String string) {
-	checkWidget ();
+	checkWidget();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if ((style & SWT.SEPARATOR) != 0) return;
-	int index = parent.indexOf (this);
-	if (index == -1) return;
 	super.setText (string);
-	updateText ((short) (index + 1));
+	String accelString = "";
+	int index = string.indexOf ('\t');
+	if (index != -1) {
+		accelString = string.substring (index, string.length());
+		string = string.substring (0, index);
+	}
+	char [] chars = fixMnemonic (string);
+	byte [] buffer = Converter.wcsToMbcs (null, chars, true);
+	int /*long*/ label = OS.gtk_bin_get_child (handle);
+	OS.gtk_label_set_text_with_mnemonic (label, buffer);
+	buffer = Converter.wcsToMbcs (null, accelString, true);
+	int /*long*/ ptr = OS.g_malloc (buffer.length);
+	OS.memmove (ptr, buffer, buffer.length);
+	int /*long*/ oldPtr = OS.GTK_ACCEL_LABEL_GET_ACCEL_STRING (label);
+	OS.GTK_ACCEL_LABEL_SET_ACCEL_STRING (label, ptr);
+	if (oldPtr != 0) OS.g_free (oldPtr);
 }
 
-void updateText (short menuIndex) {
-	if ((style & SWT.SEPARATOR) != 0) return;
-	char [] buffer = new char [text.length ()];
-	text.getChars (0, buffer.length, buffer, 0);
-	int i=0, j=0;
-	while (i < buffer.length) {
-		if (buffer [i] == '\t') break;
-		if ((buffer [j++] = buffer [i++]) == '&') {
-			if (i == buffer.length) {continue;}
-			if (buffer [i] == '&') {i++; continue;}
-			j--;
+void updateAccelerator (int /*long*/ accelGroup, boolean add) {
+	if (accelerator == 0 || !getEnabled ()) return;
+	int mask = 0;
+	if ((accelerator & SWT.ALT) != 0) mask |= OS.GDK_MOD1_MASK;
+	if ((accelerator & SWT.SHIFT) != 0) mask |= OS.GDK_SHIFT_MASK;
+	if ((accelerator & SWT.CONTROL) != 0) mask |= OS.GDK_CONTROL_MASK;
+	int keysym = accelerator & SWT.KEY_MASK;
+	int newKey = Display.untranslateKey (keysym);
+	if (newKey != 0) {
+		keysym = newKey;
+	} else {
+		switch (keysym) {
+			case '\r': keysym = OS.GDK_Return; break;
+			default: keysym = Display.wcsToMbcs ((char) keysym);
 		}
 	}
-	int str = OS.CFStringCreateWithCharacters (OS.kCFAllocatorDefault, buffer, j);
-	if (str == 0) error (SWT.ERROR_CANNOT_SET_TEXT);
-	OS.SetMenuItemTextWithCFString (parent.handle, menuIndex, str);
-	int [] outHierMenu = new int [1];
-	OS.GetMenuItemHierarchicalMenu (parent.handle, menuIndex, outHierMenu);
-	if (outHierMenu [0] != 0) OS.SetMenuTitleWithCFString (outHierMenu [0], str);
-	OS.CFRelease (str);
+	if (add) {
+		OS.gtk_widget_add_accelerator (handle, OS.activate, accelGroup, keysym, mask, OS.GTK_ACCEL_VISIBLE);
+	} else {
+		OS.gtk_widget_remove_accelerator (handle, accelGroup, keysym, mask);
+	}
 }
 }
-
