@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
 
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -195,6 +195,15 @@ public class Component implements ImageObserver, MenuContainer  {
     /** Returns true if the component is visible */
     public boolean isShowing() { return isVisible(); }
 
+    /** Returns if the component is opaque or not - Opaque components paint
+     * their entire region, while non-Opaque components paint only the 
+     * pixels they need and let the remainder show through
+     * This function is hardcoded to false to match default behaviour in
+     * java.awt, components should override this method if this isn't the case.
+     * @return true if the component is opaque
+     */
+    public boolean isOpaque() { return false; }
+
     /** Change the enabled state of the component */
     public void setEnabled(final boolean b) { 
         pEnabled = b;
@@ -492,6 +501,11 @@ public class Component implements ImageObserver, MenuContainer  {
         else
             return getPeerSize();
     }
+
+    /** Calls getSize() */
+    public Dimension size() {
+        return getSize();
+    }
     
     /**
      * Overloaded version of getSize() that uses the passed
@@ -571,6 +585,7 @@ public class Component implements ImageObserver, MenuContainer  {
 
     /** Returns the Font used on the component */
     public swingwt.awt.Font getFont() {
+	Font returnFont;
         if (!SwingWTUtils.isSWTControlAvailable(peer))
             return pFont;
         else {
@@ -579,8 +594,11 @@ public class Component implements ImageObserver, MenuContainer  {
                     retval = new swingwt.awt.Font(peer.getFont());
                 }
             });
-            return (swingwt.awt.Font) retval;
+            returnFont = (swingwt.awt.Font) retval;
         }
+	if (returnFont == null) returnFont = Font.getStubFont();
+	return returnFont;
+	
     }
     /** Sets the font used by the component */
     public void setFont(final swingwt.awt.Font f) {
@@ -906,10 +924,9 @@ public class Component implements ImageObserver, MenuContainer  {
      *  Sends action events to listeners
      */
     public void processActionEvent(int id) {
-        Iterator i = actionListeners.iterator();
         ActionEvent ae = new ActionEvent(this, id, this.getActionCommand());
-        while (i.hasNext()) {
-            ActionListener al = (ActionListener) i.next();
+        for (int i = 0; i < actionListeners.size(); i++) {
+            ActionListener al = (ActionListener) actionListeners.get(i);
             al.actionPerformed(ae);
         }
     }
@@ -931,9 +948,8 @@ public class Component implements ImageObserver, MenuContainer  {
      *  Sends mouse events to component listeners
      */
     public void processMouseEvent(MouseEvent e) {
-        Iterator i = mouseListeners.iterator();
-        while (i.hasNext()) {
-            MouseListener ml = (MouseListener) i.next();
+        for (int i = 0; i < mouseListeners.size(); i++) {
+	    MouseListener ml = (MouseListener) mouseListeners.get(i);
             if (e.eventID == MouseEvent.CLICKED) ml.mouseClicked(e);
             else if (e.eventID == MouseEvent.ENTERED) ml.mouseEntered(e);
             else if (e.eventID == MouseEvent.EXITED) ml.mouseExited(e);
@@ -1338,8 +1354,8 @@ public class Component implements ImageObserver, MenuContainer  {
             catch (IllegalAccessException e) {}
             catch (InvocationTargetException e) {}
 
-            if (!success)
-                throw new IllegalComponentStateException("SWT peer missing required methods! (?)");
+            //if (!success)
+            //    throw new IllegalComponentStateException("SWT peer missing required methods! (?)");
         }
     }
 
@@ -1423,10 +1439,43 @@ public class Component implements ImageObserver, MenuContainer  {
 
     /** Returns the component's accessible context */
     public AccessibleContext getAccessibleContext() { return accessibleContext; }
+
+    public boolean isAncestorOf(Component comp) {
+        boolean returnValue = false;
+        Container parent2 = comp.getParent();
+        while (parent2 != null) {
+             if (this == parent2) { // use equals?
+                returnValue = true;
+                break;
+             }
+             parent2 = parent2.getParent();
+        }
+        return returnValue;
+    }
+
+    
 }
 
 /*
    $Log: Component.java,v $
+   Revision 1.103  2005/03/22 15:29:49  bobintetley
+   (Mathew McBride) Component.isOpaque
+
+   Revision 1.102  2005/03/18 15:06:47  bobintetley
+   Fixed ConcurrentModification problem in generated events
+
+   Revision 1.101  2005/02/25 10:21:45  bobintetley
+   Compatibility deprecated methods (Paul Bolle)
+
+   Revision 1.100  2005/01/05 09:22:24  bobintetley
+   Updated copyright year on source
+
+   Revision 1.99  2005/01/05 08:37:11  bobintetley
+   Many compatibility fixes from David Barron
+
+   Revision 1.98  2004/11/09 16:36:23  bobintetley
+   Formatting support for JTextPane and new StyledTextDemo
+
    Revision 1.97  2004/11/02 11:07:29  bobintetley
    Fixed a number of small compatibility bugs
 
