@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
 
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -8,6 +8,12 @@
    Contact me by electronic mail: bobintetley@users.sourceforge.net
 
    $Log: Container.java,v $
+   Revision 1.56  2005/01/05 09:22:24  bobintetley
+   Updated copyright year on source
+
+   Revision 1.55  2004/11/08 10:16:37  bobintetley
+   Bounds checking for container and allows JTree to be created without a model
+
    Revision 1.54  2004/11/02 11:07:30  bobintetley
    Fixed a number of small compatibility bugs
 
@@ -511,7 +517,10 @@ public class Container extends Component {
     }
 
     public Component getComponent(int i) {
+    	if (i < comps.size())
             return (Component) comps.get(i);
+    	else
+    		return null;
     }
 
     public Component[] getComponents() {
