@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
 
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -8,6 +8,12 @@
    Contact me by electronic mail: bobintetley@users.sourceforge.net
 
    $Log: ItemEvent.java,v $
+   Revision 1.4  2005/03/09 09:41:00  bobintetley
+   (David Green) Various compatibility fixes
+
+   Revision 1.3  2005/01/05 09:22:27  bobintetley
+   Updated copyright year on source
+
    Revision 1.2  2003/12/14 09:13:38  bobintetley
    Added CVS log to source headers
 
@@ -22,6 +28,8 @@ public class ItemEvent extends AWTEvent {
     public static final int SELECTED = 0;
     public static final int DESELECTED = 1;
     
+    public static final int ITEM_STATE_CHANGED = 701;
+
     protected Object item;
     protected int stateChange;
     
