@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
 
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -8,6 +8,12 @@
    Contact me by electronic mail: bobintetley@users.sourceforge.net
 
    $Log: KeyEvent.java,v $
+   Revision 1.20  2005/01/05 09:22:27  bobintetley
+   Updated copyright year on source
+
+   Revision 1.19  2005/01/05 08:37:12  bobintetley
+   Many compatibility fixes from David Barron
+
    Revision 1.18  2004/09/01 23:20:13  dannaab
    TextAction stubbed; fix VK_UP/DOWN/etc mappings to SWT
 
@@ -59,6 +65,7 @@ public class KeyEvent extends InputEvent implements java.io.Serializable {
     private char keyChar = ' ';
     private int keyCode = 0;
     private int keyLocation;
+    private long keyWhen;
     public int eventID = 0;
     
     public static final int KEY_FIRST = 400;
@@ -487,7 +494,8 @@ public class KeyEvent extends InputEvent implements java.io.Serializable {
         this.keyCode = keyCode;
         this.keyChar = keyChar;
         this.keyLocation = keyLocation;
-	}
+	this.keyWhen = when;
+    }
 
     /** Getter for property keyChar.
      * @return Value of property keyChar.
@@ -553,4 +561,11 @@ public class KeyEvent extends InputEvent implements java.io.Serializable {
         return swt;
     }
     
+    public long getWhen() {
+	return keyWhen;
+    }
+
+    public int getKeyLocation() {
+	return keyLocation;
+    }
 }
