@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
  
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -8,6 +8,12 @@
    Contact me by electronic mail: bobintetley@users.sourceforge.net
  
    $Log: JApplet.java,v $
+   Revision 1.4  2005/01/05 09:22:30  bobintetley
+   Updated copyright year on source
+
+   Revision 1.3  2005/01/05 08:37:13  bobintetley
+   Many compatibility fixes from David Barron
+
    Revision 1.2  2004/03/22 15:28:52  bobintetley
    Complete stub Applet implementation
 
@@ -38,9 +44,5 @@ public class JApplet extends swingwt.applet.Applet {
 
     public void setContentPane(swingwt.awt.Container pane) {	    
     }
-
-    public swingwt.awt.Container getRootPane() {
-	return this;
-    }
     
 }
