@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
 
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -8,6 +8,28 @@
    Contact me by electronic mail: bobintetley@users.sourceforge.net
 
    $Log: JComboBox.java,v $
+   Revision 1.47  2005/04/20 07:46:51  bobintetley
+   Fixed JCombo to use ppeer.add instead of ppeer.setItems as it crashes on
+   GTK2 in SWT 3.1M5a
+
+   Revision 1.46  2005/04/05 16:53:28  bobintetley
+   Failed Combobox sizing fix
+
+   Revision 1.45  2005/04/04 11:13:00  bobintetley
+   Combo box fixes
+
+   Revision 1.44  2005/03/30 08:36:46  bobintetley
+   MacOSX sizing problems fixed
+
+   Revision 1.43  2005/03/09 09:41:00  bobintetley
+   (David Green) Various compatibility fixes
+
+   Revision 1.42  2005/01/05 09:22:30  bobintetley
+   Updated copyright year on source
+
+   Revision 1.41  2005/01/05 08:37:13  bobintetley
+   Many compatibility fixes from David Barron
+
    Revision 1.40  2004/11/02 11:07:30  bobintetley
    Fixed a number of small compatibility bugs
 
@@ -117,6 +139,7 @@ import swingwt.awt.event.*;
 
 import org.eclipse.swt.events.*;
 import org.eclipse.swt.widgets.*;
+//import org.eclipse.swt.custom.CCombo;
 import org.eclipse.swt.*;
 
 import java.util.*;
@@ -125,6 +148,8 @@ public class JComboBox extends swingwtx.swing.JComponent
 	implements ListDataListener {
 
     protected Combo ppeer = null;
+    //protected CCombo ppeer = null;
+    
     protected String pText = "";
     protected Vector litems = null;
     protected int pSelectedIndex = -1;
@@ -318,8 +343,12 @@ public class JComboBox extends swingwtx.swing.JComponent
 	    for (int i=0; i < litems.size(); i++) {
 		Object o = litems.get(i);
 		items[i] = o == null ? "" : o.toString();
+		// I don't like adding them one at a time, but
+		// we have no choice with 3.1M5
+		ppeer.add(items[i]);
 	    }
-	    ppeer.setItems(items);
+	    // We can't use this any more - it crashes on GTK2???
+	    //ppeer.setItems(items);
         }
     }
 
@@ -330,11 +359,33 @@ public class JComboBox extends swingwtx.swing.JComponent
 
         SwingUtilities.invokeSync(new Runnable() {
             public void run() {
-                if (SwingWTUtils.isSWTControlAvailable(ppeer))
+                if (SwingWTUtils.isSWTControlAvailable(ppeer)) {
                     ppeer.add(item.toString());
+                    fixMacOffsetProblem();
+                }
             }
         });
-
+    }
+    
+    /**
+     * MacOSX has problems with combo boxes appearing
+     * offset and weird
+     */
+    private void fixMacOffsetProblem() {
+	/*
+	 * This does not work. How the hell do we fix this?
+	 *
+        if (SwingWTUtils.isMacOSX()) {
+            // Remember where we are now
+            Control c = SwingWTUtils.getDisplay().getFocusControl();
+            // Focus the peer
+            ppeer.forceFocus();
+            // Refocus
+            if (c != null) c.forceFocus();
+  	    // Force a redraw
+	    ppeer.update();
+        }
+	*/
     }
 
     public void insertItemAt(final Object item, final int index) {
@@ -343,8 +394,10 @@ public class JComboBox extends swingwtx.swing.JComponent
         litems.add(index, item);
         SwingUtilities.invokeSync(new Runnable() {
             public void run() {
-                if (SwingWTUtils.isSWTControlAvailable(ppeer))
+                if (SwingWTUtils.isSWTControlAvailable(ppeer)) {
                     ppeer.add(item.toString(), index);
+                    fixMacOffsetProblem();
+                }
             }
         });
     }
@@ -452,24 +505,33 @@ public class JComboBox extends swingwtx.swing.JComponent
     public void setSwingWTParent(swingwt.awt.Container parent) throws Exception {
         descendantHasPeer = true;
         ppeer = new Combo(parent.getComposite(), SWT.BORDER | SWT.DROP_DOWN );
+        //ppeer = new CCombo(parent.getComposite(), SWT.BORDER | SWT.DROP_DOWN );
         
         if (litems != null) {
 	    String[] items = new String[litems.size()];
 	    for (int i=0; i < litems.size(); i++) {
 		Object o = litems.get(i);
 		items[i] = o == null ? "" : o.toString();
+		// I don't like adding them one at a time, but
+		// we have no choice with 3.1M5
+		ppeer.add(items[i]);
 	    }
-	    ppeer.setItems(items);
+	    // We can't use this any more - it crashes on GTK2???
+	    //ppeer.setItems(items);
         }
         
+	// Visible rows
+        ppeer.setVisibleItemCount(pMaxRows);
+
+
         if (pSelectedIndex != -1)
             ppeer.select(pSelectedIndex);
-
-            // Real Swing combos default to the first item if none are selected
+	// Real Swing combos default to the first item if none are selected
         else if (litems != null && litems.size() > 0) {
             pSelectedIndex = 0;
             ppeer.select(0);
         }
+        
         // If the combo is editable, default the text supplied if we have some
         if (isEditable)
             if (!pText.equals(""))
@@ -479,6 +541,8 @@ public class JComboBox extends swingwtx.swing.JComponent
         this.parent = parent;
 
         registerComboEvents();
+        fixMacOffsetProblem();
+
     }
 
     protected void registerComboEvents() {
@@ -642,4 +706,46 @@ public class JComboBox extends swingwtx.swing.JComponent
     public void contentsChanged(ListDataEvent e) {
         // TOOD: We need to update existing here
     }
+    
+    protected void selectedItemChanged() {
+        // TODO
+    }
+
+    public void setPrototypeDisplayValue(String proto) {
+        // TODO
+    }
+
+    public static interface KeySelectionManager {
+         public int selectionForKey(char aKey, ComboBoxModel aModel);
+    }
+
+    public void setKeySelectionManager(KeySelectionManager keySelectionManager) {
+        if (keySelectionManager == null) {
+            // TODO: implement setKeySelectionManager(null)
+        } 
+	else {
+            final KeySelectionManager manager = keySelectionManager;
+	    final ComboBoxModel model = getModel();
+            ppeer.addKeyListener(new org.eclipse.swt.events.KeyAdapter() {
+                 public void keyPressed(org.eclipse.swt.events.KeyEvent e) {
+                     // If ALT or CTRL is pressed, ignore this code.
+                     if ( ((e.stateMask & SWT.ALT) > 0) ||
+                         ((e.stateMask & SWT.CTRL) > 0) ) return;
+
+                     final int selection = manager.selectionForKey(e.character,model);
+                     if (selection >= 0) {
+                         SwingUtilities.invokeIn(new Runnable() {
+                              public void run() {
+                                   setSelectedItem(litems.get(selection));
+                              }
+                          }, 1);
+                     }
+                 }
+             });
+	 }
+     }
+
+     public void getKeySelectionManager() {
+         // TODO: implement getKeySelectionManager
+     }
 }
