@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
  
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -8,6 +8,12 @@
    Contact me by electronic mail: bobintetley@users.sourceforge.net
  
    $Log: JComponent.java,v $
+   Revision 1.42  2005/01/05 09:22:30  bobintetley
+   Updated copyright year on source
+
+   Revision 1.41  2005/01/05 08:37:13  bobintetley
+   Many compatibility fixes from David Barron
+
    Revision 1.40  2004/11/02 11:07:31  bobintetley
    Fixed a number of small compatibility bugs
 
@@ -588,6 +594,20 @@ public class JComponent extends swingwt.awt.Container {
         super.dispose();	
     }
 
+     public JRootPane getRootPane() {
+	JRootPane rootPane = null;
+        Container parent2 = getParent();
+        while (parent2 != null) {
+            if (parent2 instanceof JRootPane) {
+                rootPane = (JRootPane)parent2;
+                break;
+            }
+            parent2 = parent2.getParent();
+         }
+         return rootPane;
+     }
+							 
+
     /**
      * Callback for when this JComponent is added to a container.
      */
