@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
 
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -8,6 +8,35 @@
    Contact me by electronic mail: bobintetley@users.sourceforge.net
 
    $Log: JFileChooser.java,v $
+   Revision 1.34  2005/04/22 12:21:23  bobintetley
+   Fixed JFileChooser skipping bug, added LSB mount points and changed sort to
+   put directories above files.
+
+   Revision 1.33  2005/02/16 08:34:36  bobintetley
+   JFileChooser.showDialog support, JOptionPane instance support (no dialog creation at present), JSplitPane.getDividerLocation(), PixelGrabber.grabPixels(ms) throws InterruptedException, swingwtx.swing.MenuElement implemented, with JMenu* implementors, JPopupMenu.getSubElements(), JMenuBar.getSubElements(), JMenu.getSubElements()
+
+   Revision 1.32  2005/01/11 16:40:18  bobintetley
+   Code cleanup around pixmaps
+
+   Revision 1.31  2005/01/05 09:22:31  bobintetley
+   Updated copyright year on source
+
+   Revision 1.30  2004/11/05 11:31:50  bobintetley
+   getPixmap() variant that uses Class context and relative resources (Shaun Jackman)
+
+   Revision 1.29  2004/11/04 20:49:18  bobintetley
+   Relative resources breaks under Sun VMs. Gone back to the getPixmap() method
+
+   Revision 1.28  2004/11/04 20:24:21  bobintetley
+   Whoops, should at least compile before checkin :)
+
+   Revision 1.27  2004/11/04 20:16:03  bobintetley
+   Relative image resources and moved filechooser images into swingwtx.swing
+
+   Revision 1.26  2004/11/04 10:27:03  bobintetley
+   getPixmap() call to look for images on the filesystem or as a classpath
+      resource.
+
    Revision 1.25  2004/11/01 12:16:45  bobintetley
    Fixed JFileChooser drive box bug, Dialog size/display bug and a bug in the
    classpath file for Eclipse editing
@@ -266,10 +295,15 @@ public class JFileChooser extends JPanel {
         {
             checkDriveEntry("/", "Root (/)", driveBox);
             checkDriveEntry("/mnt/auto", "Automatic Mount (/mnt/auto)", driveBox);
+            checkDriveEntry("/floppy", "Floppy Disk (/floppy)", driveBox);
             checkDriveEntry("/mnt/floppy", "Floppy Disk (/mnt/floppy)", driveBox);
+            checkDriveEntry("/mediad/floppy", "Floppy Disk (/media/floppy)", driveBox);
             checkDriveEntry("/cdrom", "CDROM (/cdrom)", driveBox);
             checkDriveEntry("/cdrom1", "CDROM (/cdrom1)", driveBox);
+            checkDriveEntry("/media/cdrom", "CDROM (/media/cdrom)", driveBox);
+            checkDriveEntry("/media/cdrom1", "CDROM (/media/cdrom1)", driveBox);
             checkDriveEntry("/mnt/cdrom", "CDROM (/mnt/cdrom)", driveBox);
+            checkDriveEntry("/mnt/cdrom1", "CDROM (/mnt/cdrom1)", driveBox);
             checkDriveEntry("/mnt/dvd", "DVD ROM (/mnt/dvd)", driveBox);
             checkDriveEntry("/mnt/windows", "Windows Disk (/mnt/windows)", driveBox);
             checkDriveEntry("/mnt/win_c", "Windows Disk (/mnt/win_c)", driveBox);
@@ -323,7 +357,7 @@ public class JFileChooser extends JPanel {
 
             for (int i = 0; i < fulldir.length; i++) {
                 if (fulldir[i].getName().startsWith("."))
-                    i++; // Ignore hidden files
+                    i=i; // Ignore hidden files
                 else if (theFilter == null)
                     matches.add(fulldir[i]);
                 else if (theFilter.accept(fulldir[i]))
@@ -453,6 +487,18 @@ public class JFileChooser extends JPanel {
         else
             return showCustomOpenDialog(parent);
     }
+   
+    /** Displays the file chooser with the appropriate approval button text */
+    public int showDialog(Component parent, String approveButtonText) {
+        dialogType = OPEN_DIALOG;
+        String dt = (dialogTitle.equals("") ? "Open" : dialogTitle);
+        okButton.setText(approveButtonText);
+        customDialog = new FileChooserDialog(dt, this);
+        rescanCurrentDirectory();   // perhaps not the best place but now its consistent
+        customDialog.show();
+        post_init();
+        return dialogReturnValue;
+    }
     
     /**
      * This is a convenience method to the DirectoryDialog SWT class.  I'm sure
@@ -626,7 +672,8 @@ public class JFileChooser extends JPanel {
 
     /**
      * Sorts an array of Java <code>File</code> references
-     * into alphabetical order.
+     * into alphabetical and directory order.
+     * Crude bubblesort algorithm.
      */
     public File[] sortFiles(File[] sort) {
 
@@ -634,14 +681,35 @@ public class JFileChooser extends JPanel {
         while (madeAChange) {
             madeAChange = false;
             for (int i = 0; i < sort.length-1; i++) {
-                int pos = sort[i].toString().compareTo(sort[i+1].toString());
-                if ( pos > 0 ) {
+                // Directories take precedence, so if we have a 
+                // non-directory lower than a directory, we swap them
+                // too.
+                if ((!sort[i].isDirectory()) && sort[i+1].isDirectory()) {
                     File buf = sort[i+1];
                     sort[i+1] = sort[i];
                     sort[i] = buf;
                     buf = null;
                     madeAChange = true;
                 }
+
+		// Compare the names
+		int pos = sort[i].toString().compareTo(sort[i+1].toString());
+
+		// If moving them wouldn't put a non-directory in front
+		// of a directory, do it
+                if ( pos > 0 ) {
+		    if (sort[i].isDirectory() && !sort[i+1].isDirectory()) {
+		        // Swapping here would put a non-directory in front
+			// of a directory, so do nothing.
+		    }
+		    else {
+                        File buf = sort[i+1];
+                        sort[i+1] = sort[i];
+                        sort[i] = buf;
+                        buf = null;
+                        madeAChange = true;
+                    }
+		}
             }
         }
         return sort;
@@ -852,7 +920,7 @@ public class JFileChooser extends JPanel {
      */
     private class FileListRenderer extends JLabel implements ListCellRenderer {
 
-        ImageIcon dirIcon = new ImageIcon(getClass().getResource("/swingwtx/swing/filechooser/dir.gif"));
+    	ImageIcon dirIcon = SwingWTUtils.getPixmap(JFileChooser.class, "dir.gif");
 
         public FileListRenderer() {
             //setOpaque(true);
