@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
 
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -8,6 +8,12 @@
    Contact me by electronic mail: bobintetley@users.sourceforge.net
 
    $Log: JInternalFrame.java,v $
+   Revision 1.34  2005/01/05 09:22:31  bobintetley
+   Updated copyright year on source
+
+   Revision 1.33  2005/01/05 08:37:13  bobintetley
+   Many compatibility fixes from David Barron
+
    Revision 1.32  2004/06/11 09:17:32  bobintetley
    Various fixes and things
 
@@ -89,6 +95,7 @@ import swingwtx.swing.event.*;
 
 import org.eclipse.swt.widgets.*;
 
+import java.beans.PropertyVetoException;
 import java.util.*;
 
 /**
@@ -179,12 +186,13 @@ public class JInternalFrame extends JComponent implements WindowConstants, RootP
     public boolean isMaximizable() { return true; }
     public boolean isIconifiable() { return false; }
     public boolean isMaximum() { return true; }
-    public void setClosed(boolean b) { if (b) dispose(); }
+    
+    public void setClosed(boolean b) throws PropertyVetoException { if (b) dispose(); }
     public void setClosable(boolean b) {}
     public void setResizable(boolean b) {}
     public void setMaximizable(boolean b) {}
     public void setIconifiable(boolean b) {}
-    public void setMaximum(boolean b) {}
+    public void setMaximum(boolean b) throws PropertyVetoException {}
     public void setIcon(boolean b) { try { setSelected(false); } catch (java.beans.PropertyVetoException e) {}}
     public void toFront() { try { setSelected(true); } catch (java.beans.PropertyVetoException e) {} }
     public void toBack() {try { setSelected(false);  } catch (java.beans.PropertyVetoException e) {}}
@@ -467,4 +475,17 @@ public class JInternalFrame extends JComponent implements WindowConstants, RootP
         
     }
     
+    
+    public boolean isIcon() {
+        // TODO
+        return false;
+    }
+    
+    public void setLayer(int layer) {
+        // TODO
+    }
+    
+    protected void moveToFront() {
+        toFront();
+    }
 }
