@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
 
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -8,6 +8,18 @@
    Contact me by electronic mail: bobintetley@users.sourceforge.net
 
    $Log: JList.java,v $
+   Revision 1.43  2005/04/04 12:58:56  bobintetley
+   Fix for icons in Lists
+
+   Revision 1.42  2005/04/04 10:41:05  bobintetley
+   MacOSX sizing fixes to display vertical toolbars and JLists correctly
+
+   Revision 1.41  2005/01/05 09:22:31  bobintetley
+   Updated copyright year on source
+
+   Revision 1.40  2005/01/05 08:37:13  bobintetley
+   Many compatibility fixes from David Barron
+
    Revision 1.39  2004/09/07 23:40:32  dannaab
    Add ListDataListener support to JComboBox; fix indexOf() in JList so it supports list models that aren't extended from DefaultListModel.
 
@@ -412,9 +424,16 @@ public class JList extends swingwtx.swing.JComponent implements Scrollable, List
                 // Get a reference to the column
                 org.eclipse.swt.widgets.TableColumn tc = ppeer.getColumn(0);
                 
+                // Use the width of the peer to size the only column less
+                // 4 chars for scrollbar width - for MacOSX only since this
+                // doesn't play nice with GNOME (it seems to prefer being
+                // sized after data is added)
+                if (SwingWTUtils.isMacOSX())
+                    tc.setWidth(parent.getWidth() - SwingWTUtils.getRenderStringWidth("WWWW"));
+                
                 // Generate list data
                 if (listModel != null) {
-                    int widestListItem = 0;
+                    int maxWidth = 0;
                     for (int i = 0; i < listModel.getSize(); i++) {
                         TableItem ti = new TableItem(ppeer, 0);
                         // Get the cell renderer for this item
@@ -423,18 +442,22 @@ public class JList extends swingwtx.swing.JComponent implements Scrollable, List
 
                             // Use the text from the renderer
                             ti.setText(0, renderer.getText() );
-
-                            // Calculate needed width and update the column if 
-                            // we have a wider one
-                            int needed = SwingWTUtils.getRenderStringWidth(renderer.getText());
-                            if (needed > widestListItem) {
-                                widestListItem = needed;
-                                tc.setWidth(needed);
-                            }
                             
                             // If there's an image, render it:
                             if (renderer.getIcon() != null)
                                 ti.setImage(0, SwingWTUtils.getSWTImageFromSwingIcon(list, renderer.getIcon()));
+                            
+                            // Work out the width and adjust if necessary - this doesn't
+                            // apply to OSX, since we sized that above
+                            if (!SwingWTUtils.isMacOSX()) {
+                                int thiswidth = SwingWTUtils.getRenderStringWidth(ti.getText());
+                                if (thiswidth > maxWidth) {
+				    int iconwidth = 0;
+				    if (renderer.getIcon() != null)
+				        iconwidth = renderer.getIcon().getIconWidth();
+                                    tc.setWidth(thiswidth + iconwidth);
+				}
+                            }
 
                             // Colours
                             if (renderer.getBackground() != null)
@@ -618,5 +641,35 @@ public class JList extends swingwtx.swing.JComponent implements Scrollable, List
         // not used... let swt scrollbars handle this
         return 0;
     }
+
+    public void ensureIndexIsVisible(final int selectedIndex)
+    {
+        SwingUtilities.invokeSync(new Runnable() {
+            public void run() {
+                ppeer.select(selectedIndex);
+                ppeer.showSelection();        
+            }
+        });
+    }
+
+    public boolean isSelectedIndex(int pos) {
+        return getSelectedIndex() == pos;
+    }
+
+    public ListSelectionModel createSelectionModel() {
+        return new DefaultListSelectionModel(this);
+    }
+
+    protected void setSelectionInterval(int index, int index2) {
+        // TODO 
+    }
+    
+    public void addSelectionInterval(int index, int index2) {
+        // TODO 
+    }
+    
+    public void removeSelectionInterval(int index, int index2) {
+        // TODO 
+    }
     
 }
