@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
 
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -8,6 +8,19 @@
    Contact me by electronic mail: bobintetley@users.sourceforge.net
 
    $Log: JTabbedPane.java,v $
+   Revision 1.37  2005/03/09 09:41:00  bobintetley
+   (David Green) Various compatibility fixes
+
+   Revision 1.36  2005/02/22 17:37:48  bobintetley
+   JTabbedPane now supports add(index) methods. Few code errors fixed as a result
+   of work on 15th.
+
+   Revision 1.35  2005/01/05 09:22:31  bobintetley
+   Updated copyright year on source
+
+   Revision 1.34  2005/01/05 08:37:13  bobintetley
+   Many compatibility fixes from David Barron
+
    Revision 1.33  2004/09/01 23:20:19  dannaab
    TextAction stubbed; fix VK_UP/DOWN/etc mappings to SWT
 
@@ -148,8 +161,28 @@ public class JTabbedPane extends JComponent implements SwingConstants {
         return component;
     }
     
-    public void addTab(final String title, final Icon icon, final swingwt.awt.Component component, final String tip, final boolean enabled, boolean fromcache) {
-        
+    /** Warning: index is ignored. */
+    public swingwt.awt.Component add(swingwt.awt.Component component, int index) {
+        addTab("", component);
+        return component;
+    }
+    
+    /** Warning: index is ignored. */
+    public void add(swingwt.awt.Component component, Object modifiers, int index) {
+        addTab("", component);
+    }
+    
+    public void addTab(String title, Icon icon, swingwt.awt.Component component, String tip, boolean enabled, boolean fromcache) {
+        insertTab(title, icon, component, tip, tabs.size(), enabled, fromcache);
+    }
+    
+    public void insertTab(String title, Icon icon, Component component, String tip, int index)
+    {
+        insertTab(title, icon, component, tip, index, true, false);
+    }
+    
+    private void insertTab(final String title, final Icon icon, final Component component, final String tip, int index, final boolean enabled, boolean fromcache)
+    {        
         // Create the cache entry
         if (!fromcache) {
             TabItemCache t = new TabItemCache();
@@ -158,7 +191,7 @@ public class JTabbedPane extends JComponent implements SwingConstants {
             t.component = component;
             t.tip = tip;
             t.enabled = enabled;
-            tabs.add(t);
+            tabs.insertElementAt(t,index);
             t = null;
             return;
         }
@@ -192,9 +225,8 @@ public class JTabbedPane extends JComponent implements SwingConstants {
                 postponeLayout();
             }
         });
-        
     }
-    
+
     public Icon getIconAt(final int index) {
         if (!SwingWTUtils.isSWTControlAvailable(ppeer)) {
             TabItemCache t = (TabItemCache) tabs.get(index);
@@ -225,7 +257,21 @@ public class JTabbedPane extends JComponent implements SwingConstants {
                 }
             });
     }
-    
+   
+    public boolean isEnabledAt(final int index) {
+	 if (SwingWTUtils.isSWTControlAvailable(ppeer)) {
+ 	     SwingUtilities.invokeSync(new Runnable() {
+		 public void run() {
+		     retval = ppeer.getItem(index).getData("enabled");
+		 }
+	     });
+	     return retval.equals("true");
+	 }
+	 else {
+             TabItemCache t = (TabItemCache) tabs.get(index);
+             return t.enabled;
+	 }
+    } 
     public void setEnabledAt(final int index, final boolean b) { 
         if (SwingWTUtils.isSWTControlAvailable(ppeer)) {
             SwingUtilities.invokeSync(new Runnable() {
@@ -535,4 +581,20 @@ public class JTabbedPane extends JComponent implements SwingConstants {
         }
         return cps;
     }
+
+    public void setComponentAt(int index, Component panel)
+    {
+        if (index < tabs.size()) {
+	        TabItemCache oldItem = (TabItemCache)tabs.get(index);
+	        oldItem.component = panel;
+        }
+        else {
+            insertTab("Tab "+(index+1), null, panel, null, index, true, false);
+        }
+    }
+
+    public void setSelectedComponent(Component panel)
+    {
+        // TODO
+    }
 }
