@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
 
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -8,6 +8,21 @@
    Contact me by electronic mail: bobintetley@users.sourceforge.net
 
    $Log: JTable.java,v $
+   Revision 1.78  2005/03/17 10:06:55  bobintetley
+   getCellSelectionEnabled() had an incorrect signature
+
+   Revision 1.77  2005/03/09 09:41:00  bobintetley
+   (David Green) Various compatibility fixes
+
+   Revision 1.76  2005/02/25 10:21:45  bobintetley
+   Compatibility deprecated methods (Paul Bolle)
+
+   Revision 1.75  2005/01/05 09:22:31  bobintetley
+   Updated copyright year on source
+
+   Revision 1.74  2005/01/05 08:37:13  bobintetley
+   Many compatibility fixes from David Barron
+
    Revision 1.73  2004/09/01 23:20:19  dannaab
    TextAction stubbed; fix VK_UP/DOWN/etc mappings to SWT
 
@@ -217,16 +232,37 @@
 
 package swingwtx.swing;
 
-import swingwtx.swing.table.*;
-import swingwtx.swing.table.TableColumn;
-import swingwtx.swing.event.*;
+import java.util.EventObject;
+import java.util.HashMap;
+import java.util.Vector;
+
+import org.eclipse.swt.SWT;
+import org.eclipse.swt.widgets.Event;
+import org.eclipse.swt.widgets.Listener;
+import org.eclipse.swt.widgets.Table;
+import org.eclipse.swt.widgets.TableItem;
+
 import swingwt.awt.Component;
+import swingwt.awt.Dimension;
+import swingwt.awt.Point;
 import swingwt.awt.event.MouseEvent;
-
-import org.eclipse.swt.widgets.*;
-import org.eclipse.swt.*;
-
-import java.util.*;
+import swingwtx.swing.event.CellEditorListener;
+import swingwtx.swing.event.ChangeEvent;
+import swingwtx.swing.event.ListSelectionEvent;
+import swingwtx.swing.event.ListSelectionListener;
+import swingwtx.swing.event.TableColumnModelEvent;
+import swingwtx.swing.event.TableColumnModelListener;
+import swingwtx.swing.event.TableModelEvent;
+import swingwtx.swing.event.TableModelListener;
+import swingwtx.swing.table.DefaultTableCellRenderer;
+import swingwtx.swing.table.DefaultTableColumnModel;
+import swingwtx.swing.table.DefaultTableModel;
+import swingwtx.swing.table.JTableHeader;
+import swingwtx.swing.table.TableCellEditor;
+import swingwtx.swing.table.TableCellRenderer;
+import swingwtx.swing.table.TableColumn;
+import swingwtx.swing.table.TableColumnModel;
+import swingwtx.swing.table.TableModel;
 
 
 
@@ -480,7 +516,7 @@ public class JTable extends swingwtx.swing.JComponent implements TableModelListe
     public void setColumnSelectionAllowed(boolean b) { this.columnSelection = b; }
     public boolean getRowSelectionAllowed(boolean b) { return rowSelection; }
     public void setCellSelectionEnabled(boolean b) { cellSelectionEnabled = b; modelDirty = true; refreshTable(); }
-    public boolean getCellSelectionEnabled(boolean b) { return cellSelectionEnabled; }
+    public boolean getCellSelectionEnabled() { return cellSelectionEnabled; }
     public boolean getColumnSelectionAllowed(boolean b) { return columnSelection; }
     /** NOT IMPLEMENTED - HARD RETURNS A COLOUR VALUE */
     public swingwt.awt.Color getSelectionBackground() { return swingwt.awt.Color.BLUE; }
@@ -645,6 +681,11 @@ public class JTable extends swingwtx.swing.JComponent implements TableModelListe
         return header;
     }
 
+    public void setTableHeader(JTableHeader header) {
+	// TODO: implement null table header
+	this.header = header;
+    }
+
     /**
      * Once a parent component receives an "add" call for a child, this being
      * the child, this should be called to tell us to instantiate the peer
@@ -1269,5 +1310,34 @@ public class JTable extends swingwtx.swing.JComponent implements TableModelListe
     public void removeColumn(TableColumn tableColumn) {
         getColumnModel().removeColumn(tableColumn);
     }
+    
+    public Dimension getPreferredScrollableViewportSize()
+    {
+        // TODO actual value probably less
+        return getSize();
+    }
+    
+    public void setPreferredScrollableViewportSize(Dimension dimension)
+    {
+        // TODO
+    }
+    
+    public boolean isCellEditable(int row, int column)
+    {
+        return model.isCellEditable(row, column);
+    }
+    
+    public boolean editCellAt(int row, int column, EventObject e) {
+        // TODO
+        return false;
+    }
+
+    public boolean editCellAt(int row, int column) {
+        return editCellAt(row, column, null);
+    }
+    
+    protected int columnAtPoint(Point p) {
+        return columnModel.getColumnIndexAtX(p.x);
+    }
 
 }
