@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
  
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -8,6 +8,12 @@
    Contact me by electronic mail: bobintetley@users.sourceforge.net
  
    $Log: JToolBar.java,v $
+   Revision 1.35  2005/04/04 10:41:05  bobintetley
+   MacOSX sizing fixes to display vertical toolbars and JLists correctly
+
+   Revision 1.34  2005/01/05 09:22:32  bobintetley
+   Updated copyright year on source
+
    Revision 1.33  2004/10/30 20:11:57  bobintetley
    Code cleanup
 
@@ -361,6 +367,13 @@ public class JToolBar extends JComponent implements SwingConstants {
             add((Component) b);
             return b;
         }
+
+	// If the toolbar is vertical, the Mac doesn't like it
+	// so we have to use a real button instead of a toolitem
+	// (effectively a composite with SWT RowLayout)
+	if (SwingWTUtils.isMacOSX() && orientation == VERTICAL) {
+	    add((Component) b);
+	}
         
         // Map the JButton to a fake action to use the platform toolbars
         JButtonMappedAction x = new JButtonMappedAction(b);
