@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
 
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -8,6 +8,18 @@
    Contact me by electronic mail: bobintetley@users.sourceforge.net
 
    $Log: JTree.java,v $
+   Revision 1.42  2005/03/09 09:41:00  bobintetley
+   (David Green) Various compatibility fixes
+
+   Revision 1.41  2005/01/05 09:22:32  bobintetley
+   Updated copyright year on source
+
+   Revision 1.40  2005/01/05 08:37:13  bobintetley
+   Many compatibility fixes from David Barron
+
+   Revision 1.39  2004/11/08 10:16:37  bobintetley
+   Bounds checking for container and allows JTree to be created without a model
+
    Revision 1.38  2004/09/30 02:00:25  dannaab
    Notify text component's document on setText() of new content.
 
@@ -101,6 +113,7 @@ import org.eclipse.swt.*;
 
 import swingwtx.swing.event.*;
 import swingwtx.swing.tree.*;
+import swingwtx.swing.plaf.*;
 
 import java.util.*;
 
@@ -500,6 +513,26 @@ public class JTree extends swingwtx.swing.JComponent implements TreeModelListene
         return new DefaultTreeSelectionModel();
     }
 
+    /**
+     *  Convert the given tree node value to text - The
+     *  default implementation simply returns 
+     *  <code>(value == null)?"":value.toString(<code>,
+     *  however sublcasses may overide this method to 
+     *  provide alternative behaviour  Used by renderers
+     *  to provide a text representation of the value
+     *
+     *  @param value The value of the tree node
+     *  @param selected if the tree node is selected
+     *  @param expanded if the tree node is expanded
+     *  @param leaf it the tree node is a leaf
+     *  @param row the row of the display
+     *  @param hasFocus if the tree node has focus
+     *  @return the text representation of the tree node
+     */
+     public String convertValueToText(Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
+	  return (value == null) ? "" : value.toString();
+     }
+
     /** Scrolls the Tree to show the selected path */
     public void scrollPathToVisible(final TreePath t) {
         SwingUtilities.invokeSync(new Runnable() {
@@ -665,6 +698,9 @@ public class JTree extends swingwtx.swing.JComponent implements TreeModelListene
         // Don't bother drawing if no peer has been created
         if (!SwingWTUtils.isSWTControlAvailable(ppeer)) return;
 
+        // No point if we haven't got a model either      
+        if (model == null) return;
+        
         SwingUtilities.invokeSync(new Runnable() {
             public void run() {
 
@@ -673,6 +709,7 @@ public class JTree extends swingwtx.swing.JComponent implements TreeModelListene
 
                 // ROOT BASED RENDER:
                 // =====================================
+                
                 Object root = model.getRoot();
                 if (root != null) {
                     if(isRootVisible){
@@ -867,5 +904,65 @@ cellRenderer = renderer; }
     }
 
 
+    public boolean isExpanded(int j)
+    {
+        // TODO 
+        return false;
+    }
+    
+    public int getRowForLocation(int x, int y)
+    {
+        // TODO
+        return 0;
+    }
+    
+    public void setSelectionRow(int row)
+    {
+        // TODO
+    }
+    
+    public boolean isEditing()
+    {
+        // TODO
+        return false;
+    }
+    
+    public void stopEditing()
+    {
+        // TODO
+    }
+    
+    public void addTreeWillExpandListener(TreeWillExpandListener tree)
+    {
+        // TODO
+    }
+    
+    protected void setInvokesStopCellEditing(boolean b)
+    {
+        // TODO
+    }
+    
+    protected void startEditingAtPath(TreePath newChildPath)
+    {
+        // TODO
+    }
+    
+    protected TreeUI getUI()
+    {
+        // TODO
+        return null;
+    }
+    
+    public void treeDidChange()
+    {
+        // TODO
+    }
+    
+    public void setSelectionInterval(int i, int rowCount)
+    {
+        // TODO
+    }
+
+
 }
 
