@@ -1,6 +1,6 @@
 /*
    SwingWT
-   Copyright(c)2003-2004, R. Rawson-Tetley
+   Copyright(c)2003-2005, R. Rawson-Tetley
 
    For more information on distributing and using this program, please
    see the accompanying "COPYING" file.
@@ -31,20 +31,54 @@ import java.util.*;
  */
 public abstract class SwingWTUtils {
 
+	/**
+	 * Whether or not to output information about SwingWT to stdout when
+	 * the library is loaded.
+	 */
     private static boolean isDebug = true;
+    
+    /** SwingWT version */
+    private final static String VERSION = "0.87 (020605)";
+    
+    /**
+     * Whether or not SWTExceptions thrown on the event loop are
+     * output to stdout.
+     */
     public static boolean showInternalSWTExceptions = false;
     
-    private final static String VERSION = "0.85 (021104)";
+    /**
+     * Set this flag to true if you are compiling SwingWT natively for use
+     * with GCJ. It forces the getPixmap() call to look for files on the
+     * file system instead of as compiled in resources.
+     */
+    public static boolean useFileSystemPixmaps = false;
+    /**
+     * If we are using file system pixmaps, this is the path to the
+     * pixmaps directory. This should be correct for most *nix flavours
+     * but you might need to change it if your distribution keeps
+     * pixmaps in a funky place. It SHOULD have a trailing slash.
+     */
+    public static String PIXMAP_DIRECTORY = "/usr/share/pixmaps/";
+    /**
+     * If the app is natively compiled, this is the subdirectory of
+     * PIXMAP_DIRECTORY that SwingWT's internal components (tree, filechooser, etc.)
+     * will look in to find images. It should NOT have a trailing or leading slash
+     */
+    public static String SWINGWT_PIXMAP_SUBDIR = "swingwt"; 
+    
+    /**
+     * The copyright message output on startup
+     */
     private final static String COPYRIGHT = "This is SwingWT (http://swingwt.sourceforge.net)\n" +
-                                            "Version: " + VERSION + "\n" +
-                                            "Copyright(c)2003-2004, R.Rawson-Tetley and other contributors.\n\n" +
+                                            "Version: " + VERSION + "[SWT " + SWT.getPlatform() + " " + SWT.getVersion() + "]\n" +
+                                            "Copyright(c)2003-2005, R.Rawson-Tetley and other contributors.\n\n" +
                                             "This library is distributed in the hope that it will be useful,\n" +
                                             "but WITHOUT ANY WARRANTY; without even the implied warranty of\n" + 
                                             "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU\n" +
                                             "Lesser General Public Licence for more details.";
     
+    /** The SWT display */
     private static Display display = null;
-    
     /** Whether we have a dispatch thread going */
     private static boolean eventsDispatching = false;
     /** Who owns the event dispatcher */
@@ -58,19 +92,23 @@ public abstract class SwingWTUtils {
     private static int retardationms = 2000;
     /** Window reference count. If it reaches 0 then we end the dispatch thread */
     private static int windowReferenceCount = 0;
-    /** Whether or not to use SWT fast virtuals. 
-     *  NOTE: As at M8 there is an issue with Win32 environments 
-     *  and virtual tables so it defaults to off for Windows.
+    /** 
+     * Whether or not to use SWT fast virtuals. 
+     * NOTE: As at M8 there is an issue with Win32 environments 
+     * and virtual tables so it defaults to off for Windows.
      */
     private static boolean useSWTFastVirtualTables = !isWindows();
-    
+    /** A temp directory SwingWT can use (not used at present) */
     private static String tempDirectory = System.getProperty("user.home") + File.separator + "tmp" + File.separator + "swingwt";
+    /** 
+     * The height of the MacOS X menubar in pixels - this is used by awt.Window as calling setLocation(0,0)
+     * on MacOSX can end up hiding the window decoration. This value determines where SwingWT thinks
+     * the MacOSX screen actually starts. 
+     * */
+    public static int MACOSX_MENUBAR_BOTTOM = 20;
     
     public static String getVersion() { return VERSION; }
     
-    /** The height of the MacOS X menubar in pixels */
-    public static int MACOSX_MENUBAR_BOTTOM = 20;
-    
     public static synchronized void incrementWindowReferences() {
         windowReferenceCount++;    
     }
@@ -283,6 +321,47 @@ public abstract class SwingWTUtils {
     public static synchronized void stopEventDispatchRunning() {
         eventsDispatching = false;
     }
+
+    /**
+     * Used for retrieving images in a friendly way to both GCJ and
+     * VMs. If the useFileSystemPixmaps flag is set (for a native build),
+     * this will retrieve the image from the file system, looking in
+     * the pixmapSubdir of PIXMAP_DIRECTORY. If the flag is not set,
+     * then the resourcePath is used to look on the classpath for the
+     * image.
+     *
+     * @param filename The name of the image file
+     * @param pixmapSubdir The subdirectory of /usr/share/pixmaps/ to look for the image in
+     * @param resourcePath The URL to the classpath resource (eg: /com/mydomain/)
+     * @return An ImageIcon containing the pixmap
+     */
+    public static ImageIcon getPixmap(String filename, String pixmapSubdir, String resourcePath) {
+    	if (useFileSystemPixmaps) {
+    		if (!pixmapSubdir.endsWith("/")) pixmapSubdir += "/";
+    		return new ImageIcon(PIXMAP_DIRECTORY + pixmapSubdir + filename);
+    	}
+    	else {
+    		if (!resourcePath.endsWith("/")) resourcePath += "/";
+    		return new ImageIcon(filename.getClass().getResource(resourcePath + filename));
+    	}
+	}
+    
+    /**
+     * Similar to the other getPixmap, this overloaded version uses a
+     * class for context and retrieves the resource through a relative
+     * link if we aren't using filesystem pixmaps.
+     * 
+     * @param context The class to use to load the resource from
+     * @param filename The resource's filename
+     * @return An ImageIcon containing the pixmap
+     */
+    public static ImageIcon getPixmap(Class context, String filename) {
+    	if (useFileSystemPixmaps) {
+    		return new ImageIcon(PIXMAP_DIRECTORY + SWINGWT_PIXMAP_SUBDIR + "/" + filename);
+    	}
+    	else
+    		return new ImageIcon(context.getResource(filename));
+    }
     
     /**
      * @param url The path to inspect.
@@ -491,7 +570,16 @@ public abstract class SwingWTUtils {
                     gc = new org.eclipse.swt.graphics.GC(getDisplay().getActiveShell());
 
                 org.eclipse.swt.graphics.Point p = gc.stringExtent(text + "W");
+		
                 int height = p.y;
+		
+		// MacOSX gets it wrong
+		if (isMacOSX()) {
+	            double dh = (double) height;
+		    double nh = dh * 1.5;
+		    height = (int) nh;
+		}
+
                 gc.dispose();
                 gc = null;
                 if (im != null) im.dispose();
@@ -669,6 +757,70 @@ public abstract class SwingWTUtils {
 
 /**
     $Log: SwingWTUtils.java,v $
+    Revision 1.107  2005/04/22 12:21:23  bobintetley
+    Fixed JFileChooser skipping bug, added LSB mount points and changed sort to
+    put directories above files.
+
+    Revision 1.106  2005/04/04 10:41:06  bobintetley
+    MacOSX sizing fixes to display vertical toolbars and JLists correctly
+
+    Revision 1.105  2005/03/30 08:36:46  bobintetley
+    MacOSX sizing problems fixed
+
+    Revision 1.104  2005/03/23 08:14:56  bobintetley
+    Fixes to AWTSwingWrapper to set the parent correctly (fixes AWT components
+    not getting added to forms) and to register component events.
+
+    Revision 1.103  2005/03/18 15:06:50  bobintetley
+    Fixed ConcurrentModification problem in generated events
+
+    Revision 1.102  2005/03/09 09:41:00  bobintetley
+    (David Green) Various compatibility fixes
+
+    Revision 1.101  2005/02/22 17:47:40  bobintetley
+    Fixed ConcurrentModification problem with menu items
+
+    Revision 1.100  2005/01/25 10:07:31  bobintetley
+    Font family support and GraphicsEnvironment implementation
+
+    Revision 1.99  2005/01/21 08:22:30  bobintetley
+    Patch from Andreas Kiebach to add a new JTable implementation with
+    a couple of bug fixes to Text components and for Dialogs to immediately
+    grab the focus
+
+    Revision 1.98  2005/01/13 12:35:52  bobintetley
+    Upgraded to 3.1M3 - the speed difference to GTK2 applications is simply
+    jaw dropping. Also verified and retested the native build script for
+    GTK2 and patched the SWT code for all platforms natively compiling with GCJ.
+
+    Revision 1.97  2005/01/11 16:40:18  bobintetley
+    Code cleanup around pixmaps
+
+    Revision 1.96  2005/01/05 09:22:32  bobintetley
+    Updated copyright year on source
+
+    Revision 1.95  2005/01/05 08:37:13  bobintetley
+    Many compatibility fixes from David Barron
+
+    Revision 1.94  2004/12/17 08:34:12  bobintetley
+    Fix to text components to make setCaretPosition() work
+
+    Revision 1.93  2004/11/10 08:54:45  bobintetley
+    Caret fix and improvements to JTextPane
+
+    Revision 1.92  2004/11/09 16:36:24  bobintetley
+    Formatting support for JTextPane and new StyledTextDemo
+
+    Revision 1.91  2004/11/05 11:31:51  bobintetley
+    getPixmap() variant that uses Class context and relative resources (Shaun Jackman)
+
+    Revision 1.90  2004/11/04 20:16:04  bobintetley
+    Relative image resources and moved filechooser images into swingwtx.swing
+
+    Revision 1.89  2004/11/04 10:27:03  bobintetley
+    getPixmap() call to look for images on the filesystem or as a classpath
+       resource.
+
     Revision 1.88  2004/11/02 11:28:56  bobintetley
     Versioned off for 0.85
 
